from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, User, Ac0Comment, Ac0Conversation
from tcsoa.gen.ActiveCollaboration._2020_12.ActiveCollaboration import CreateOrUpdateCommentRequest, CreateOrUpdateConversationRequest, ReadResponse
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class ActiveCollaborationService(TcService):

    @classmethod
    def createOrUpdateComment(cls, request: List[CreateOrUpdateCommentRequest]) -> ServiceData:
        """
        This operation creates or updates a comment.
        
        Use cases:
        &bull;    A user replies to a comment in a conversation.
        &bull;    A user updates an existing comment in the conversation
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateComment',
            library='ActiveCollaboration',
            service_date='2020_12',
            service_name='ActiveCollaboration',
            params={'request': request},
            response_cls=ServiceData,
        )

    @classmethod
    def createOrUpdateConversation(cls, request: CreateOrUpdateConversationRequest) -> ServiceData:
        """
        This operation creates or updates a conversation.
        
        Use cases:
        &bull;    A user creates a new public conversation on any Teamcenter object.
        Any user can create or update a conversation.  Conversations are typically created through Active Workspace on
        the conversations panel.  A conversation must have one or more source objects.  A source object can be any
        Teamcenter WorkspaceObject.  
        
        &bull;    A user creates a new private conversation on any Teamcenter object.
        Any user can create or update a conversation.  Conversations are typically created through Active Workspace on
        the conversations panel.  A conversation must have one or more source objects.  A source object can be any
        Teamcenter WorkspaceObject.  Private conversations must contain at least one participant.  A participant is a
        Teamcenter User object.
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateConversation',
            library='ActiveCollaboration',
            service_date='2020_12',
            service_name='ActiveCollaboration',
            params={'request': request},
            response_cls=ServiceData,
        )

    @classmethod
    def deleteComment(cls, objsToDelete: List[Ac0Comment]) -> ServiceData:
        """
        This operation deletes one or more Ac0Comment object(s).
        """
        return cls.execute_soa_method(
            method_name='deleteComment',
            library='ActiveCollaboration',
            service_date='2020_12',
            service_name='ActiveCollaboration',
            params={'objsToDelete': objsToDelete},
            response_cls=ServiceData,
        )

    @classmethod
    def deleteConversation(cls, objsToDelete: List[Ac0Conversation]) -> ServiceData:
        """
        This operation deletes one or more Ac0Conversation object(s).
        """
        return cls.execute_soa_method(
            method_name='deleteConversation',
            library='ActiveCollaboration',
            service_date='2020_12',
            service_name='ActiveCollaboration',
            params={'objsToDelete': objsToDelete},
            response_cls=ServiceData,
        )

    @classmethod
    def getReadInfo(cls, users: List[User], objects: List[BusinessObject]) -> ReadResponse:
        """
        Given a list of users and a list of objects, this operation returns the users and objects where the user(s) do
        not have read access to the object(s).
        
        Use cases:
        &bull;    A user creating a conversation needs to know which source objects the given participants do not have
        read access to.  
        
        When creating conversations in Active Workspace, a user can select one or more participants for a conversation
        on one or more source objects.  The user creating the conversation needs to know which participants do not have
        read access to all source objects.   
        
        For each participant that does not have read access to one or more source objects, the operation returns a
        mapping of the participant and the source objects for which that participant does not have read access.    
        
        Using this information, a user creating the conversation can then either know that a participant will not be
        able to see the message they are writing, or they may do what is needed have the permissions changed for the
        participant(s) or the source object(s).
        
        &bull;    A user creating a conversation needs to know which participants do not have read access for the
        source objects on the conversation.
        
        When creating conversations in Active Workspace, a user can select one or more participants for a conversation
        on one or more source objects.  The user creating the conversation needs to know which source objects are not
        readable by all participants.   
        
        For each source object on which any participant does not have read access, the operation returns the source
        object and the participants that do not have read access to that object. 
        
        Using this information, a user creating the conversation can then either know that a participants will not be
        able to see the message they are writing, or they may do what is needed have the permissions changed for the
        participant(s) or the source object(s).
        """
        return cls.execute_soa_method(
            method_name='getReadInfo',
            library='ActiveCollaboration',
            service_date='2020_12',
            service_name='ActiveCollaboration',
            params={'users': users, 'objects': objects},
            response_cls=ReadResponse,
        )

    @classmethod
    def manageSubscriptions(cls, conversations: List[Ac0Conversation], sourceObjects: List[BusinessObject], subscriptionFlag: bool) -> ServiceData:
        """
        This operation subscribes or unsubscribes the current user to a list of Ac0Conversation or source objects. The
        source objects are expected to be WorkspaceObject type. If user subscribe to an Ac0Conversation object, then
        when new comments are added to the conversation as replies the user will be notified.  If user subscribe to a
        WorkspaceObject, then when new conversations are created for that WorkspaceObject or new comments are added as
        replies to existing Conversations on that WorkspaceObject, the user will be notified.
        
        Use cases:
        &bull;    Follow or Subscribe to a Conversation
        When a user clicks on the "follow" button in an expanded conversation, they will receive notifications when
        replies are made to the conversation.  A conversation is any Ac0Conversation.
        
        &bull;    Unfollow or Unsubscribe to a Conversation
        If a user has subscribed to a conversation by clicking on the Follow button on the Conversation, they will now
        see an "Unfollow" button.  Click the Unfollow button to stop receiving notifications when replies are made to
        the conversation.  A conversation is any Ac0Conversation
        
        &bull;    Follow or Subscribe to Conversations for a source object
        When a user clicks on the "follow" button in the Conversation panel, they will receive notifications for any
        replies to existing conversations, as well as for any new conversations created, where the currently selected
        business object is a source object on the conversation.  A source object can be any Teamcenter WorkspaceObject.
        
        &bull;    Unfollow or Unsubscribe to Conversations for a source object
        If a user has subscribed to conversations for a source object, they will now see an "Unfollow" button next to
        the selected source object in the conversation panel.  Click the "Unfollow" button to stop receiving
        notifications for conversation activity related to the selected source object.  A source object can be any
        Teamcenter WorkspaceObject.
        """
        return cls.execute_soa_method(
            method_name='manageSubscriptions',
            library='ActiveCollaboration',
            service_date='2020_12',
            service_name='ActiveCollaboration',
            params={'conversations': conversations, 'sourceObjects': sourceObjects, 'subscriptionFlag': subscriptionFlag},
            response_cls=ServiceData,
        )



from __future__ import annotations

from enum import Enum


class SoaEPMAction(Enum):
    """
    SoaEPMAction
    """
    SOA_EPM_assign_action_2013_05 = 'SOA_EPM_assign_action_2013_05'
    SOA_EPM_start_action_2013_05 = 'SOA_EPM_start_action_2013_05'
    SOA_EPM_remove_attachment_action_2013_05 = 'SOA_EPM_remove_attachment_action_2013_05'
    SOA_EPM_approve_action_2013_05 = 'SOA_EPM_approve_action_2013_05'
    SOA_EPM_reject_action_2013_05 = 'SOA_EPM_reject_action_2013_05'
    SOA_EPM_promote_action_2013_05 = 'SOA_EPM_promote_action_2013_05'
    SOA_EPM_demote_action_2013_05 = 'SOA_EPM_demote_action_2013_05'
    SOA_EPM_refuse_action_2013_05 = 'SOA_EPM_refuse_action_2013_05'
    SOA_EPM_assign_approver_action_2013_05 = 'SOA_EPM_assign_approver_action_2013_05'
    SOA_EPM_notify_action_2013_05 = 'SOA_EPM_notify_action_2013_05'
    SOA_EPM_no_action_2013_05 = 'SOA_EPM_no_action_2013_05'
    SOA_EPM_fail_action_2013_05 = 'SOA_EPM_fail_action_2013_05'
    SOA_EPM_complete_action_2013_05 = 'SOA_EPM_complete_action_2013_05'
    SOA_EPM_skip_action_2013_05 = 'SOA_EPM_skip_action_2013_05'
    SOA_EPM_suspend_action_2013_05 = 'SOA_EPM_suspend_action_2013_05'
    SOA_EPM_resume_action_2013_05 = 'SOA_EPM_resume_action_2013_05'
    SOA_EPM_undo_action_2013_05 = 'SOA_EPM_undo_action_2013_05'
    SOA_EPM_abort_action_2013_05 = 'SOA_EPM_abort_action_2013_05'
    SOA_EPM_perform_action_2013_05 = 'SOA_EPM_perform_action_2013_05'
    SOA_EPM_add_attachment_action_2013_05 = 'SOA_EPM_add_attachment_action_2013_05'


class SoaEPMSupportingValues(Enum):
    """
    SoaEPMSupportingValues
    """
    SOA_EPM_no_decision_2013_05 = 'SOA_EPM_no_decision_2013_05'
    SOA_EPM_approve_2013_05 = 'SOA_EPM_approve_2013_05'
    SOA_EPM_reject_2013_05 = 'SOA_EPM_reject_2013_05'
    SOA_EPM_unset_2013_05 = 'SOA_EPM_unset_2013_05'
    SOA_EPM_completed_2013_05 = 'SOA_EPM_completed_2013_05'
    SOA_EPM_unable_to_complete_2013_05 = 'SOA_EPM_unable_to_complete_2013_05'
    SOA_EPM_true_2013_05 = 'SOA_EPM_true_2013_05'
    SOA_EPM_false_2013_05 = 'SOA_EPM_false_2013_05'
    SOA_EPM_no_error_2013_05 = 'SOA_EPM_no_error_2013_05'



from __future__ import annotations

from typing import List
from tcsoa.gen.Query._2007_01.SavedQuery import RetrieveQueryCriteriaResponse, SaveQueryCriteriaInfo
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class SavedQueryService(TcService):

    @classmethod
    def reorderSavedQueryCriterias(cls, queryCriteriaNames: List[str]) -> ServiceData:
        """
        Reorder the saved Query Criterias in the MySavedSearches List:
          The new order of query criteria names specified in the input list will be stored in the MySavedSearches list. 
          The input list should contain only existing query criteria names 
          If a query criteria name in the list is not located, it will not be stored in the list. 
          The number of entries in the input list should match the number entries in the MySavedSearches list.
        
        Exceptions:
        >- When input argument is not specified
        - When input vector is empty
        
        """
        return cls.execute_soa_method(
            method_name='reorderSavedQueryCriterias',
            library='Query',
            service_date='2007_01',
            service_name='SavedQuery',
            params={'queryCriteriaNames': queryCriteriaNames},
            response_cls=ServiceData,
        )

    @classmethod
    def retrieveQueryCriterias(cls, queryCriteriaNames: List[str]) -> RetrieveQueryCriteriaResponse:
        """
        Retrieve the information on the saved search by the search name.
        """
        return cls.execute_soa_method(
            method_name='retrieveQueryCriterias',
            library='Query',
            service_date='2007_01',
            service_name='SavedQuery',
            params={'queryCriteriaNames': queryCriteriaNames},
            response_cls=RetrieveQueryCriteriaResponse,
        )

    @classmethod
    def saveQueryCriterias(cls, queryCriterias: List[SaveQueryCriteriaInfo]) -> ServiceData:
        """
        Saves a saved search with search name, query name, entry names, and entry values. If search name is not
        provided, the criteria keys or the criteria values size is 0, or the criteria keys size does not equal to the
        criteria values size, or if error happens while creating the saved search, the related error information will
        be added to the error stack. If the search criteria size is no more than 0, ServiceException will throw out of
        this service. The created saved search objects will be returned.
        
        Use cases:
        User selects a query and fills in some criterias, and then save the search from thin client with a search name.
        """
        return cls.execute_soa_method(
            method_name='saveQueryCriterias',
            library='Query',
            service_date='2007_01',
            service_name='SavedQuery',
            params={'queryCriterias': queryCriterias},
            response_cls=ServiceData,
        )

    @classmethod
    def deleteQueryCriterias(cls, queryCriteriaNames: List[str]) -> ServiceData:
        """
        Delete saved searches with given names.
        
        Use cases:
        Delete specified saved searches.
        """
        return cls.execute_soa_method(
            method_name='deleteQueryCriterias',
            library='Query',
            service_date='2007_01',
            service_name='SavedQuery',
            params={'queryCriteriaNames': queryCriteriaNames},
            response_cls=ServiceData,
        )



from __future__ import annotations

from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class RegisteredCallbackObjectsResponse(TcBaseObj):
    """
    The response is the object containing registered callback function details and the service data
    
    :var parameters: The registered callback functions details for given input type.
    :var serviceData: Standard ServiceData member.
    """
    parameters: List[RegisteredCallbackParam] = ()
    serviceData: ServiceData = None


@dataclass
class RegisteredCallbackParam(TcBaseObj):
    """
    The data objects which contains the  details of customized registered callback functions.
    
    :var type: The registered callback function type. Supported types are: "Normalization_Callback",
    "MFG_ValidationChecksCallback" or any customized callback type used to register callback.
    :var library: The customization DLL name.
    :var name: The customization callback name.
    :var functionName: The registered callback function name.
    """
    type: str = ''
    library: str = ''
    name: str = ''
    functionName: str = ''



from tcsoa.gen.Internal_AuditManager._2012_09.services import ProcessHistoryManagementService as imp0
from tcsoa.base import TcService


class ProcessHistoryManagementService(TcService):
    getProcssHistoryAuditRecords = imp0.getProcssHistoryAuditRecords



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BOMLine
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class VariantManagementService(TcService):

    @classmethod
    def applyRollupVariantConfiguration(cls, bomlines: List[BOMLine], considerWindowSVR: bool) -> ServiceData:
        """
        The applyRollupVariantConfiguration operation applies overlay product configurator authored configuration on
        the BOM Window by generating the rolled up variant expression from input BOM Lines. If the considerWindowSVR
        input is set to true, it considers the existing Variant Rule applied on the BOM Window along with the rolled up
        expression. BOMWindow must have effective configurator context for this operation to be successful.
        
        Use cases:
        It allows users to work in the context of product structure lines that truly coexist when variants are
        configured.Users won't have to cascade variant conditions from child to parents. This will reduce the
        complexity in variant conditions managed by the customers and at the same time make the overall process more
        efficient.
        """
        return cls.execute_soa_method(
            method_name='applyRollupVariantConfiguration',
            library='Internal-StructureManagement',
            service_date='2019_06',
            service_name='VariantManagement',
            params={'bomlines': bomlines, 'considerWindowSVR': considerWindowSVR},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.Internal.Mmv._2012_09.SpatialStructureManagement import GetNodeBBoxIn, GetNodeBBoxResponse, AcquireSpatialHierarchyResponse, CreateMmvCursorResponse, IsSpatialHierarchyIn, GetSpatialCellsReadTicketsResponse, SearchScope, AcquireSpatialHierarchyIn, ReleaseSpatialHierarchyIn, SpatialHierCellInfo, IsSpatialHierarchyLatestResponse
from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class SpatialStructureManagementService(TcService):

    @classmethod
    def getSpatialCellsReadTickets(cls, input: SpatialHierCellInfo) -> GetSpatialCellsReadTicketsResponse:
        """
        This is the primary method used during rendering via MMV technology.  The visualization client traverses a
        spatial hierarchy (.mmv file) downloaded from the server that contains an array of cells, and submits
        individual cells that it expects to be visible to the server for configuration and resolution into occurrences
        by calling this service.  The JT part and transformation information from the occurrences is returned and then
        loaded and rendered by the client.  This operation is called continuously during the VGR rendering process.  As
        the actual geometry is loaded the visibility occlusion is refined and more 'GetSpatialCellsReadTickets' calls
        are made, but the amount of cells decreases over time along with the service calls as the scene stabilizes. 
        This rendering approach is referred to as Visibility Guided Rendering (VGR), where a spatial hierarchy is used
        to determine which geometry is visible for a given visibility request.
        This service depends on first running the MMV index harvester process to create the Fnd0SpatialHierarchy
        dataset and populate the MMV occurrence tables on the server.  The dataset contains an .mmv file that
        represents the spatial hierarchy the viewer loads and traverses during rendering.  The database tables contain
        a mapping between the cells of the spatial hierarchy and the occurrences in the structure that was harvested. 
        The harvested structure can be unconfigured, or partially configured.  Thus the spatial hierarchy can represent
        an unconfigured or partially configured model.  This method basically takes a list of cells to be rendered by
        the client, looks up their corresponding occurrences in Teamcenter, configures BOMLines for the occurrences
        referenced by these cells applying the appropriate configuration rules of the current BOMWindow, and returns
        these occurrences to the client along with the JT file information.  The basic idea is to only configure the
        occurrences that are visible, thus reducing overall work and time required to render the large model.
        This operation gets the information of configured occurrences that are located in an array of cells.  In
        addition to the cell information, this operation also takes as input one or more cursor objects that are
        created with the 'createMMVCursor' service.  The cursors contain the visibility scopes to be applied when
        configuring these cells.  The visibility scope refers to a set of occurrences (parts or assemblies) within
        which visible geometry is to be displayed.  Providing the visibility scope to the server enables the client to
        render a specific subassembly directly from the spatial hierarchy (.mmv file) without first knowing the
        occurrences within that subassembly.  The client renders from the spatial hierarchy regardless if the entire
        model or a specific subset of the model is being set visible.  During rendering, the server will only return
        occurrences from the 'GetSpatialCellsReadTickets' calls that are contained within the visibility scoping set
        specified by the client.  If the client wishes to set the entire model visible, the scoping set is left blank. 
        If the client is to render a specific subassembly, the scoping set is the occurrence thread path of the
        specific subassembly.
        Any combination of cursor objects in the input cursor array can be applied on each cell, and this combination
        is represented by a 32 bit signed integer for this cell.  Each bit in the 32 bit integer, except the highest
        sign bit, indicates whether or not the corresponding cursor in the cursor array is enabled for the cell.  For
        example, integration value of 0x0003 would indicate that the first and second cursor in the input cursor array
        is enabled.  At most 31 cursor objects can be handled by any single operation.  
        The returned occurrence information includes BOMLine, transform, JT file location and ticket, and to what
        cursors each occurrence belongs using a similar bitmask mechanism as described above.
        
        Use cases:
        The visualization client performs the following sequence when viewing a configured assembly from Teamcenter
        using MMV technology:
            1.    Construct a BOMWindow with the appropriate configuration for the structure of interest
            2.    Determine if MMV based rendering can be used for this structure by invoking 
        'acquireSpatialHierarchy' and passing the top line of the structure
                a.    If MMV possible, a view lock is applied on the Fnd0SpatialHierarchy dataset attached to the
        ItemRevision that represents the top line of the structure, and a read file ticket is returned for the spatial
        hierarchy (*.mmv) file for subsequent download.
                b.    If MMV rendering is not possible, the appropriate error is returned to the client.  This can
        happen most often if the structure was not harvested, or if the configuration of interest does not match the
        configuration that was harvested.
            3.    Prior to rendering, the visualization client needs to set the view frustum for the visibility request
        of interest.  This is done by first calling the 'getNodeBBox' service for all the occurrences that are to be
        set visible in order to setup the view.  The bounding box information returned is processed and the view setup
        properly prior to rendering.
            4.    Prior to rendering from the spatial hierarchy the client must first create an mmv cursor via the
        'createMmvCursor', and specify the visibility scoping set in the cursor.  If the entire model is to be set
        visible, the visibility scope is not specified.  If a specific set of occurrences are to be set visible (e.g.
        one or more subassemblies), the visibility scoping set must be set to the occurrence thread paths representing
        the occurrences in scope (e.g. the subassemblies and parts to render).
            5.    Rendering is now ready to begin.  Rendering consists of traversing the spatial hierarchy and
        resolving cells to visible occurrences, and occurrences are loaded.   Spatial cells are supplied to the
        'GetSpatialCellsReadTickets' along with spatial hierarchy dataset uid, BOMWindow and the cursor object.  The
        initial cursor contains the visibility scoping set.  Configured occurrences are returned from the call.
            6.    The client downloads the JT parts referenced by the visible occurrences returned, loads the geometry,
        and applies the transformation.  The geometry is rendered.  The VGR algorithm reevaluates the visibility based
        on the real loaded geometry, and determines additional cells that need to be loaded.
            7.    The process returns to step 5 above and repeats continuously until the rendering stabilizes and the
        picture is complete.  The calls to 'GetSpatialCellsReadTickets' eventually stop when the rendering completes.
        """
        return cls.execute_soa_method(
            method_name='getSpatialCellsReadTickets',
            library='Internal-Mmv',
            service_date='2012_09',
            service_name='SpatialStructureManagement',
            params={'input': input},
            response_cls=GetSpatialCellsReadTicketsResponse,
        )

    @classmethod
    def isSpatialHierarchyLatest(cls, input: IsSpatialHierarchyIn) -> IsSpatialHierarchyLatestResponse:
        """
        This operation is called periodically (via polling) by the viewer when it is running in Kiosk mode to see if
        there is a new MMV spatial hierarchy (.mmv file) available for loading.  This service operation checks if the
        supplied Fnd0SpatialHierarchy dataset version is the latest or not.  Since the MMV harvester is intended to run
        constantly, a new version of the spatial hierarchy may become available for the client at any time.  This
        allows the viewer to detect when a newer version of the spatial hierarchy is available, and update itself to
        the latest model indexed by the MMV harvester.  If a new Fnd0SpatialHierarchy dataset version is created by the
        harvester and found by this method, the viewer will unload the current spatial hierarchy, download the new .mmv
        file, load the new spatial hierarchy, and rerender the model.
        
        Use cases:
        The Visualization client can be placed in a kiosk mode, where it polls the server for updated spatial hierarchy
        datasets.  In this mode, the viewer will periodically call the 'isSpatialHierarchyLatest' service to see if a
        new spatial hierarchy file is available.  If one is found, the viewer will update to the new spatial hierarchy
        file and reestablish its state as follows:
            1.    Viewer periodically checks for new Fnd0SpatialHierarchy dataset version by calling
        'isSpatialHierarchyLatest'.  If none found, it waits and calls again later.  However, if found the following
        steps then occur.
            2.    Viewer captures its current state by saving a vis session file locally containing all the snapshots
        and authored data relevant for the session.  This includes a snapshot of the current 3D scene, along with the
        configuration information for the structure currently loaded.  The configuration information is stored as a
        VisStructureContext object in Teamcenter.
            3.    Viewer unloads the spatial hierarchy and 3D model by closing the 3D document
            4.    Viewer loads the vis session back up, which causes the structure to be reopened and configured the
        way it had been configured prior to the update.  The 'createBOMsFromRecipes' service is used to reestablish the
        structure configuration.
            5.    The viewer applies the snapshot that represented the scene that was present in the kiosk prior to the
        spatial hierarchy update.  This causes the occurrence references within the snapshot to resolve, which then
        results in loading the appropriate product Structure into the viewer.  Once the product structure is loaded,
        the visibility request is made.  For subassembly visibility requests, the cursor is created with the visibility
        scoping set, and MMV rendering commences to create the scene.  
            6.    The viewer goes back to kiosk mode, going back to whatever it was doing before the update.
        """
        return cls.execute_soa_method(
            method_name='isSpatialHierarchyLatest',
            library='Internal-Mmv',
            service_date='2012_09',
            service_name='SpatialStructureManagement',
            params={'input': input},
            response_cls=IsSpatialHierarchyLatestResponse,
        )

    @classmethod
    def releaseMmvCursor(cls, mmvCursors: List[BusinessObject]) -> ServiceData:
        """
        This operation releases a runtime cursor object in the tcserver that was previously created by the
        'createMmvCursor' function.  This service is intended to be called when the client has completed rendering a
        particular visibility request and the 'GetSpatialCellsReadTickets' calls have completed.
        
        Use cases:
        The Visualization client releases the cursor once the user closes the assemblies or completes the visualization
        rendering cycle for subassemblies.  
        The visualization client performs the following sequence when viewing a configured assembly from Teamcenter
        using MMV technology:
            1.    Construct a BOMWindow with the appropriate configuration for the structure of interest
            2.    Determine if MMV based rendering can be used for this structure by invoking 
        'acquireSpatialHierarchy' and passing the top line of the structure
                a.    If MMV possible, a view lock is applied on the Fnd0SpatialHierarchy dataset attached to the
        ItemRevision that represents the top line of the structure, and a read file ticket is returned for the spatial
        hierarchy (*.mmv) file for subsequent download.
                b.    If MMV rendering is not possible, the appropriate error is returned to the client.  This can
        happen most often if the structure was not harvested, or if the configuration of interested does not match the
        configuration that was harvested.
            3.    To start rendering from the spatial hierarchy the client must first create an mmv cursor via the
        'createMmvCursor', and specify the visibility scoping set in the cursor.  If the entire model is to be set
        visible, the visibility scope is not specified.  If a specific set of occurrences are to be set visible (e.g.
        one or more subassemblies), the visibility scoping set must be set to the occurrence thread paths representing
        the occurrences in scope (e.g. the subassemblies and parts to render).
            4.    The visualization client renders the geometry using MMV by making a series of
        'GetSpatialCellsReadTickets' calls to configure and resolve spatial cells into occurrences that are
        subsequently loaded and rendered.
            5.    When the rendering of the subassembly is complete or when the 3D document window is closed or if
        viewer itself is closed the client calls the 'releaseMmvCursor' to free the cursor resources.
        """
        return cls.execute_soa_method(
            method_name='releaseMmvCursor',
            library='Internal-Mmv',
            service_date='2012_09',
            service_name='SpatialStructureManagement',
            params={'mmvCursors': mmvCursors},
            response_cls=ServiceData,
        )

    @classmethod
    def releaseSpatialHierarchy(cls, input: ReleaseSpatialHierarchyIn) -> ServiceData:
        """
        Since the MMV harvester is running continuously on the server, the MMV spatial index can be updated at any
        time, even when clients are using it.  Thus, clients obtain a read lock on the spatial hierarchy dataset (.mmv)
        when it is opened via the 'acquireSpatialHierarchy' service.  However, when the client is no longer using the
        spatial hierarchy, the read lock should be released.  This operation releases a lock on the set of dataset
        versions in the current tcserver process.  This reduces the lock counts of these dataset versions by 1.  The
        visualization client calls this method whenever the 3D document that is rendered via MMV technology is closed,
        or if the viewer itself is closed.  
        Releasing the view lock on the dataset version allows the harvester to delete this version in its next run when
        no lock remains on the dataset version.  Provisions are also made for the harvester to check the tcserver
        session process table and release locks when process go away for unexpected reasons such as crashes.
        
        Use cases:
        The visualization client performs the following sequence when viewing a configured assembly from Teamcenter
        using MMV technology:
            1.    Construct a BOMWindow with the appropriate configuration for the structure of interest
            2.    Determine if MMV based rendering can be used for this structure by invoking 
        'acquireSpatialHierarchy' and passing the top line of the structure
                a.    If MMV possible, a view lock is applied on the Fnd0SpatialHierarchy dataset attached to the
        ItemRevision that represents the top line of the structure, and a read file ticket is returned for the spatial
        hierarchy (*.mmv) file for subsequent download.
                b.    If MMV rendering is not possible, the appropriate error is returned to the client.  This can
        happen most often if the structure was not harvested, or if the configuration of interest does not match the
        configuration that was harvested.
            3.    Render the model using MMV technology
            4.    If 3D document window closed or if viewer itself is closed, call the 'releaseSpatialHierarchy'
        service to release the read lock the client has on the Fnd0SpatialHierarchy dataset.
        """
        return cls.execute_soa_method(
            method_name='releaseSpatialHierarchy',
            library='Internal-Mmv',
            service_date='2012_09',
            service_name='SpatialStructureManagement',
            params={'input': input},
            response_cls=ServiceData,
        )

    @classmethod
    def createMmvCursor(cls, scope: SearchScope) -> CreateMmvCursorResponse:
        """
        This operation creates a runtime cursor object that is later used with the 'GetSpatialCellsReadTickets' service
        to maintain server state (progress) during rendering when a series of 'GetSpatialCellsReadTickets' calls are
        made.  That is, the next 'GetSpatialCellsReadTickets' call starts where the last call left off via the runtime
        cursor.   When the cursor is first created, the visibility scope at the visualization client side must be
        specified.  The visibility scope refers to a set of occurrences (parts or assemblies) within which visible
        geometry is to be displayed.  Providing the visibility scope to the server enables the client to render one or
        more specific subassemblies directly from the spatial hierarchy (.mmv file) without first knowing the
        occurrences within these subassemblies.  The client renders from the spatial hierarchy regardless if the entire
        model or a specific subset of the model is being set visible.  During rendering, the server will only return
        occurrences from the 'GetSpatialCellsReadTickets' calls that are contained within the visibility scoping set
        specified by the client.  If the client wishes to set the entire model visible, the scoping set is left blank. 
        If the client is to render one or more specific subassemblies, the scoping set is the occurrence thread paths
        of these specific subassemblies.
        The visibility scoping set can only be specified during cursor creation via this call.  If the visibility
        changes, a new cursor must be created prior to the 'GetSpatialCellsReadTickets' calls made during the rendering
        process.
        
        
        Use cases:
        The visualization client performs the following sequence when viewing a configured assembly from Teamcenter
        using MMV technology:
            1.    Construct a BOMWindow with the appropriate configuration for the structure of interest
            2.    Determine if MMV based rendering can be used for this structure by invoking 
        'acquireSpatialHierarchy' and passing the top line of the structure
                a.    If MMV possible, a view lock is applied on the Fnd0SpatialHierarchy dataset attached to the
        ItemRevision that represents the top line of the structure, and a read file ticket is returned for the spatial
        hierarchy (*.mmv) file for subsequent download.
                b.    If MMV rendering is not possible, the appropriate error is returned to the client.  This can
        happen most often if the structure was not harvested, or if the configuration of interested does not match the
        configuration that was harvested.
            3.    To commence rendering from the spatial hierarchy the client must first create an mmv cursor via the
        'createMmvCursor', and specify the visibility scoping set in the cursor.  If the entire model is to be set
        visible, the visibility scope is not specified.  If a specific set of occurrences are to be set visible (e.g.
        one or more subassemblies), the visibility scoping set must be set to the occurrence thread paths representing
        the occurrences in scope (e.g. the subassemblies and parts to render).
            4.    The visualization client renders the geometry using MMV by making a series of
        'GetSpatialCellsReadTickets' calls to configure and resolve spatial cells into occurrences that are
        subsequently loaded and rendered.
        """
        return cls.execute_soa_method(
            method_name='createMmvCursor',
            library='Internal-Mmv',
            service_date='2012_09',
            service_name='SpatialStructureManagement',
            params={'scope': scope},
            response_cls=CreateMmvCursorResponse,
        )

    @classmethod
    def acquireSpatialHierarchy(cls, input: AcquireSpatialHierarchyIn) -> AcquireSpatialHierarchyResponse:
        """
        The visualization client calls this service when a configured structure is opened from Teamcenter to determine
        whether or not the model can be rendered using MMV technology.  To use MMV technology, the structure must have
        been harvested and an MMV spatial index created, and the configuration of the structure of interest must match
        the configurations that were harvested.  If these conditions are met, the service returns dataset information
        and a read ticket to the spatial hierarchy  (.mmv) file that is read and used for MMV rendering; otherwise, an
        error is returned.
        This operation returns the latest version of Fnd0SpatialHierarchy dataset attached to the input BOMLine 
        Itemrevision in the case MMV rendering is possible (i.e. the structure has been harvested).  The dataset
        returned is locked for viewing and the associated information including ImanFile, original file name, and
        ticket information is returned.  The lock is applied to prevent this version from being removed by the mmv
        index harvester that may be schedule to be run periodically to update the spatial index.  That is, when
        multiple clients are viewing a structure using MMV technology, they must not be adversely affected by the MMV
        harvester when it updates the spatial index.  This is accomplished by having the clients create a read lock on
        the Fnd0SpatialHierarchy dataset , indicating to the harvester that it must not delete MMV indexes that are
        currently in use, but rather it must create a new version of the spatial index dataset.  A map array is
        returned to map the returned dataset version array to the input BOMLine array.  
        The service also validates whether or not the spatial hierarchy file (.mmv) can be used for the current
        structure being viewed.  The MMV index harvester supports unconfigured harvesting as well as preconfigured
        harvesting to limit the scope of what was harvested if desired.  The Fnd0SpatialHierarchy dataset created by
        the harvester contains a record of the configuration of the structure used to generate the spatial hierarchy. 
        The configuration of the current bomwindow (passed in as the top line of the structure) is checked to make sure
        it matches the configuration that was used to generate the spatial index.   If the configurations match, the
        Fnd0SpatialHierarchy dataset information will be returned.  If they do not match, an error is returned
        indicating that particular configuration was not harvested.
        
        Use cases:
        The visualization client performs the following sequence when viewing a configured assembly from Teamcenter
        using MMV technology:
            1.    Construct a bomwindow with the appropriate configuration for the structure of interest
            2.    Determine if MMV based rendering can be used for this structure by invoking 
        'acquireSpatialHierarchy' and passing the top line of the structure
                a.    If MMV possible, a view lock is applied on the Fnd0SpatialHierarchy dataset attached to the
        ItemRevision that represents the top line of the structure, and a read file ticket is returned for the spatial
        hierarchy (*.mmv) file for subsequent download by the client.
                b.    If MMV rendering is not possible, the appropriate error is returned to the client.  This can
        happen most often if the structure was not harvested, or if the configuration of interest does not match the
        configuration that was harvested.
            3.    Render the geometry based on user action using MMV if possible.  If MMV possible, download the load
        the .mmv file and render from the spatial hierarchy.  If not possible, use traditional large model
        visualization rendering technology.
        """
        return cls.execute_soa_method(
            method_name='acquireSpatialHierarchy',
            library='Internal-Mmv',
            service_date='2012_09',
            service_name='SpatialStructureManagement',
            params={'input': input},
            response_cls=AcquireSpatialHierarchyResponse,
        )

    @classmethod
    def getNodeBBox(cls, input: GetNodeBBoxIn) -> GetNodeBBoxResponse:
        """
        This operation gets bounding box (bbox) information for a set of input node objects from the MMV index on the
        server. Each object is represented by a series of UIDs representing its occurrence thread path. The returned
        'NodeBBox' array is parallel to the input node array.  A 'NodeBBox' structure may contain multiple bounding
        boxes, meaning that there can be multiple node bbox records for the node as the node may represent a
        subassembly.
        The advantage of this approach is that the bounding boxes can be rapidly computed from the MMV index and
        positioned in model space, so the performance is very high.
        
        Use cases:
        The Visualization client must properly set the view frustum when prompted by the user to view a subassembly by
        clicking the node in the tree structure to ensure the proper zoom level for the assembly.  In addition, the
        viewer must support actions such as fit all in cases where the product structure is not yet known.  This
        requires the bounding box information be positioned in model space for the nodes to be set visible.  
        After the Fnd0SpatialHierarchy (.mmv file) has been acquired using 'acquireSpatialHierarchy' operation, the
        file is loaded.  When a visibility request is received from the user (e.g. by clicking one or more nodes in the
        assembly tree), the viewer must get the bounding box for these specified nodes as follows:
            1.    Get the occurrence thread uid paths from the tree structure.
            2.    Invoke the 'getNodeBBox' operation by supplying it with the occurrence thread uid paths, and use the
        returned bounding box to set the view frustum. 
            3.    Determine which cells from the spatial hierarchy need to be configured based on their position
        relative to the bounding box information.
            4.    The client traverses only the bounding box scoped cells of the spatial hierarchy during rendering,
        and submits those to the server for configuration and resolution.
        """
        return cls.execute_soa_method(
            method_name='getNodeBBox',
            library='Internal-Mmv',
            service_date='2012_09',
            service_name='SpatialStructureManagement',
            params={'input': input},
            response_cls=GetNodeBBoxResponse,
        )



from __future__ import annotations

from tcsoa.gen.Cad._2007_12.DataManagement import CreateOrUpdatePartsPref
from tcsoa.gen.Cad._2008_06.DataManagement import CreateOrUpdatePartsResponse
from typing import List
from tcsoa.gen.Cad._2012_09.DataManagement import PartInfo
from tcsoa.base import TcService


class DataManagementService(TcService):

    @classmethod
    def createOrUpdateParts(cls, partInput: List[PartInfo], pref: CreateOrUpdatePartsPref) -> CreateOrUpdatePartsResponse:
        """
        'CreateOrUpdateParts' allows the user to create or update a set of Items, ItemRevisions, and Datasets (CAD
        concept of a Part includes these Teamcenter objects). The service first attempts to validate the existence of
        the Item, ItemRevision, and Dataset. If the Item already exist,s but the ItemRevision does not, then a new
        initial ItemRevision is created-any existing ItemRevisions are not revised. If the Item and ItemRevision
        already exist, but the Dataset does not, then only the Dataset is created. If the Dataset exists, a new version
        will be added to the existing series. If any of the objects exist but the input attribute values that differ
        from those already set, attempts are made to update the values if possible. If no Item object reference or
        ItemRevision object references are specified then a new Item and ItemRevision and related objects will be
        created. All objects created and updated will be returned in the 'ServiceData'. All partial errors will contain
        the 'clientIDs' for all items related to the error message, i.e. if a dataset encounters an error, then the ID
        for that error will be the item client ID concatenated with the item revision id concatenated with the dataset
        client ID, all separated by semi-colons ( ; ).
        
        Use cases:
        User wants to create a new CAD Part (item, item revision, and dataset). User fills in the 'CreateOrUpdateInput'
        structure with the information for the item and item ievision.
        
        User wants to create an item and item revision with one or more datasets. Client fills in the
        'CreateOrUpdateInput' structure with the information for the item and item revision from the user. Client also
        fills in one or more 'DatasetInfos' with the information about the datasets to create from the user. Upon
        return from the service, the client will extract the 'FileTickets' from the 'DatasetOutputs' and upload the
        data files for the datasets using FMS. Once the uploads have completed, then the client will use the
        'DatasetCommitInfos' to attach the upload files to the datasets using the Core Services 'commitDatasetFiles'.
        
        User wants to modify properties on an item, item revision or dataset. Client fills in the 'CreateOrUpdateInput'
        structure with the new information for the item or item revision. Client fills in the new information in a
        'DatasetInfo' structure then invokes the service using an existing item or item revision or dataset.
        
        User wants to create or update an existing item and/or item revision with User created Form object. The client
        fills in the necessary data to create or update an item and/or item revision. The client also specifies
        'itemExtraObjectInfo' and/or 'itemRevisionExtraObjectInfo' containing the form and relation to be used to
        attach the form to another object.
        
        User wants to add objects to a dataset. This can be done as Extra Objects or using the NamedReference feature.
        The user fills in the information necessary to create or update a dataset. The user can then specify
        'ExtraObjectInfo' data for attaching forms or 'NamedReferenceObjectInfo' for other objects.
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateParts',
            library='Cad',
            service_date='2012_09',
            service_name='DataManagement',
            params={'partInput': partInput, 'pref': pref},
            response_cls=CreateOrUpdatePartsResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class SearchBOEInputInfo(TcBaseObj):
    """
    List of Plant BOP lines and a map of search criteria.
    
    :var inputScopes: A list of Business object representing Plant Bill of Process  (Plant BOP) lines.
    :var searchCriteria: A map (string, string) of search criteria. The key of the map is the criterion and its value
    is the information about that criterion.
    """
    inputScopes: List[BusinessObject] = ()
    searchCriteria: SearchCriteriaMap = None


@dataclass
class SearchBOEResponse(TcBaseObj):
    """
    The response contains a list of all the BOE objects which matches the search criteria. The following partial errors
    may be returned:
    (i) 253037: The Search criteria does not contain the closure rule name property.
    (ii) 253041: The given closure rule is invalid.
    (iii) 200181: This type of search is not supported.
    
    :var resultObjects: The list of the result BOE objects matched as per the criteria. The Bill of Equipment stations
    hold location information and contain the equipment required for manufacturing.
    :var serviceData: The Service Data.
    """
    resultObjects: List[BusinessObject] = ()
    serviceData: ServiceData = None


"""
A map (string, string) where key represents the type of search criterion and the value represents additional information to perform the search.
For ex.
1. Search using closure rule: the key is "Closure_Rule" and the value is the name of the closure rule "MFGProcessAreaConsumption".
2. Search using occurrence type: the key is "Valid_Occ_Type" and the value is "MEWorkarea".
"""
SearchCriteriaMap = Dict[str, str]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BOMLine, BusinessObject, POM_object
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class ApplyBOMMarkupParam(TcBaseObj):
    """
    The parameter to apply BOMMarkup.
    
    :var bomLine: The BOMLine object whose Markup is to be applied. The MarkupChanges for the Occurrence of the line
    will not be applied.
    :var recursive: Flag that indicates whether to apply all Markups for the structure under the specified line.
    :var evaluate: Flag that indicates whether to check write access ('true') or actually modify ('false') the
    structure.
    """
    bomLine: BOMLine = None
    recursive: bool = False
    evaluate: bool = False


@dataclass
class CreateBOMMarkupOutput(TcBaseObj):
    """
    Created markup change and the containing markup object.
    
    :var markupChange: The created change object.
    :var markup: The markup object that contains the markup change.
    :var bomLine: The bomline related to the markup object.
    """
    markupChange: POM_object = None
    markup: POM_object = None
    bomLine: BOMLine = None


@dataclass
class CreateBOMMarkupParam(TcBaseObj):
    """
    Structure for the information to create a Markup Change object.
    
    :var bomLine: The selected line for the Markup action.
    :var markupType: Type of Markup to create: 
    - 1 for 'ADD'
    - 2 for 'REMOVE'
    - 4 for 'SUBSTITUTE'
    - 8 for 'REPLACE'
    - 16 for 'PROPERTY'
    - 32 for 'COMMENT'
    
    
    :var inputObject: Item or Item Revision to be used for substitute, replace or addition. It can be a variant
    expression object for a property value change to legacy variant condition.
    :var context: Context of the edit. It is the BOMView object for substitute and replace. Not used for any other type
    of Markup Change.
    :var propertyName: Name of the property to be modified. When used by 'ADD' Markup, it is one of the following
    properties: find number, quantity and occurrence type.
    :var propertyValue: String value of the property. If used by 'ADD' Markup, it is one of the property values to
    initialize the added line for the properties specified by propertyName.
    :var note: Plain text note for the change. It can be a proposed change that is not covered by supported Markup
    types, for example, to insert a level.
    """
    bomLine: BOMLine = None
    markupType: int = 0
    inputObject: BusinessObject = None
    context: BusinessObject = None
    propertyName: str = ''
    propertyValue: str = ''
    note: str = ''


@dataclass
class CreateBOMMarkupResponse(TcBaseObj):
    """
    The response of creating BOM Markups.
    
    :var createBOMMarkupsList: Set of Markups created given the input.
    :var serviceData: The service data of the operation.
    """
    createBOMMarkupsList: List[CreateBOMMarkupOutput] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.Internal.DocumentManagement._2008_06.DispatcherManagement import ConfigInput, ConfigOutput
from tcsoa.gen.Internal.DocumentManagement._2008_06.DocumentControl import IRDCResponse
from tcsoa.gen.BusinessObjects import ItemRevision
from typing import List
from tcsoa.base import TcService


class DispatcherManagementService(TcService):

    @classmethod
    def getConfigurations(cls, svcConfigInput: List[ConfigInput]) -> ConfigOutput:
        """
        The Document Management Render system uses the DispatcherServiceConfig business object to determine what
        Dispatcher Service to invoke for performing the file translation for a given source dataset type to derived
        dataset type.  This operation returns a list of DispatcherServiceConfig business objects deployed in the
        Teamcenter system based on the 'ConfigInput' list structure (contains source data set type, derived dataset
        type and service available).  The 'ConfigInput' structure is used to filter for source dataset type, derived
        dataset type, and service available for the DispatcherServiceConfig business object retrieval.  To retrieve all
        DispatcherServiceConfig business objects, set 'ConfigInput' structure source dataset type name and derived
        dataset type name set to empty string (use double quotes to specify empty string),  and service available set
        to negative or minus 1.  There is no default.
        
        The 'DocumentControl' service 'getIRDCs' operation normally invoke first to obtain the Item Revision Definition
        Configure (IRDC) setting of source dataset type and derived dataset type dataset associated to the ItemRevision
        in order to determine the derived dataset to be generated for the specify source dataset type.
        
        Use cases:
        Use case 1: Retrieve a specific DispatcherServiceConfig business object
        The following example has a DispatcherServiceConfig  business object defining previewservice to be used to
        translate MSWordX (its named reference file .docx) dataset to PDF (the generated .pdf file will be stored as
        named reference file) dataset type:
        - DispatcherServiceConfig  Name=MSWordXTo_PDF_previewservice, Source dataset type name=MSWordX, Derived dataset
        type name=PDF, Dispatcher service=previewservice, Sort Order=1, Service Available=true
        
        
        
        The Document Management Render system (Teamcenter RenderMgtTranslator) can retrieve an instance of
        DispatcherServiceConfig business object filter for source dataset type=MSWordX, derived dataset type=PDF and
        service available=1.  This will return a single list of DispatcherServiceConfig business object containing a
        single object defined in the above example.
        
        Use case 2: Retrieve DispatcherServiceConfig business objects contains service available set to true
        The following example has two DispatcherServiceConfig business objects defining pdfgenerator or previewservice
        to be used to translate MSWordX (its named reference file .docx) dataset to PDF (the generated .pdf file will
        be stored as named reference file) dataset type:
        - DispatcherServiceConfig  Name=MSWordXTo_PDF_pdfgenerator, Source dataset type name=MSWordX, Derived dataset
        type name=PDF, Dispatcher service=pdfgenerator, Sort Order=2, Service Available=true
        
        
        
        - DispatcherServiceConfig  Name=MSWordXTo_PDF_previewservice, Source dataset type name=MSWordX, Derived dataset
        type name=PDF, Dispatcher service=previewservice, Sort Order=1, Service Available=true
        
        
        
        The Document Management Render system (Teamcenter RenderMgtTranslator) can retrieve DispatcherServiceConfig
        business objects filter for source dataset type=MSWordX, derived dataset type=PDF and service available=1. 
        This will return a single list of DispatcherServiceConfig business object containing two business objects
        (DispatcherServiceConfig Name=MSWordXTo_PDF_pdfgenerator, and MSWordXTo_PDF_previewservice) defined in the
        above example.  The Document Management Render system will select the DispatcherServiceConfig business object
        with higher sort order value if there are duplicate objects with the same values specified for the combination
        of source dataset type and derived dataset type.
          
        The system allows defining multiple DispatcherServiceConfig business objects for the same types so the
        Administrator can have backup translators available and select which one to be used without deleting and
        creating the DispatcherServicConfig business objects.
        
        Use case 3: Retrieve all DispatcherServiceConfig business objects
        The following example has three DispatcherServiceConfig business objects defining pdfgenerator or
        previewservice to be used to translate MSWordX (its named reference file .docx) dataset to PDF (the generated
        .pdf file will be stored as named reference file) dataset type. Previewsevice to be used to translate MSExcelX
        (its named reference file .xlsx) dataset type to PDF dataset type:
        
        - DispatcherServiceConfig  Name= MSWordXTo_PDF_pdfgenerator, Source dataset type name= MSWordX, Derived dataset
        type name=PDF, Dispatcher service= pdfgenerator, Sort Order=2, Service Available=true
        
        
        
        - DispatcherServiceConfig  Name= MSWordXTo_PDF_previewservice, Source dataset type name= MSWordX, Derived
        dataset type name=PDF, Dispatcher service= previewservice, Sort Order=1, Service Available=false
        
        
        
        - DispatcherServiceConfig  Name= MSWordXTo_PDF_previewservice, Source dataset type name= MSExcelX, Derived
        dataset type name=PDF, Dispatcher service= previewservice, Sort Order=1, Service Available=true
        
        
        
        The Document Management Render system (Teamcenter RenderMgtTranslator) can retrieve all DispatcherServiceConfig
        objects by filtering for source dataset type=empty string, derived dataset type=empty string and service
        available=negative or minus 1.  This will return a single list of DispatcherServiceConfig business object
        containing all three business objects as defined in the above example.
        """
        return cls.execute_soa_method(
            method_name='getConfigurations',
            library='Internal-DocumentManagement',
            service_date='2008_06',
            service_name='DispatcherManagement',
            params={'svcConfigInput': svcConfigInput},
            response_cls=ConfigOutput,
        )


class DocumentControlService(TcService):

    @classmethod
    def getIRDCs(cls, itemRevs: List[ItemRevision]) -> IRDCResponse:
        """
        This operation is used by the Document Management Render system (Teamcenter RenderMgtTranslator) prior to
        performing a render of the given ItemRevision objects. In order to perform the render of an ItemRevision, the
        system needs to obtain the Item Revision Definition Configuration (IRDC) settings for the ItemRevision first.
        The IRDC object specifies necessary information for the render, such as the source and derived dataset types.
        
        There is only a single IRDC object returned per ItemRevision. The return vector is the same length as the input
        vector, with a 1 to 1 relation between the two lists.
        
        This operation is internal and only intended to be used by the Document Management Render system. The IRDC
        object should be treated as read only. Only the Business Modeler IDE (BMIDE) application is allowed to create
        or modify IRDC settings.
        
        Use cases:
        Retrieve render data for one or more ItemRevision business objects.
        
        The Document Management Render system is given a list of ItemRevision objects for rendering. First it must
        invoke the getIRDCs operation in order to determine what the source and derived dataset types are for each
        ItemRevision. Once the system has the IRDC objects, it can use the getConfigurations operation to get the
        DispatcherServiceConfig object for each translation, and then perform the translation.
        """
        return cls.execute_soa_method(
            method_name='getIRDCs',
            library='Internal-DocumentManagement',
            service_date='2008_06',
            service_name='DocumentControl',
            params={'itemRevs': itemRevs},
            response_cls=IRDCResponse,
        )



from __future__ import annotations

from tcsoa.gen.Internal.AWS2._2015_03.FullTextSearch import GetIndexedObjectsResponse, GetAMImpactedObjectsResponse, GetModifiedObjectsToSyncResponse
from tcsoa.gen.Internal.AWS2._2015_03.DataManagement import GetStyleSheetIn, GetStyleSheetResponse
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService
from datetime import datetime


class DataManagementService(TcService):

    @classmethod
    def getStyleSheet(cls, processEntireXRT: bool, input: List[GetStyleSheetIn]) -> GetStyleSheetResponse:
        """
        This operation returns the stylesheets and the required data to present that stylesheet for each input object.
        
        Use cases:
        Use Case 1: Open an object in ActiveWorkspace.
        When an object is selected in ActiveWorkspace, the user may choose to invoke the open operation. When that
        operation is executed the getStylesheet call is invoked and the Stylesheet is processed in order to display the
        data to the user.
        
        Use Case 2: List view with Summary
        When an object is selected in the ActiveWorkspace navigator, when in list view with summary mode, the
        getStyleSheet SOA operation is invoked, and the returned data is parsed/processed in order to present the data
        to the user in the Summary panel.
        
        Use Case 3: Show Object Info
        When an object is selected in ActiveWorkspace, a user may choose the "Show Object Info" command.  When
        executed, a panel slides out from the right hand side of the application, and the selected objects data is
        presented.  In order to populate the panel, the getStylesheet call is invoked and the returned data is parsed
        and processed in order to build the UI.
        """
        return cls.execute_soa_method(
            method_name='getStyleSheet',
            library='Internal-AWS2',
            service_date='2015_03',
            service_name='DataManagement',
            params={'processEntireXRT': processEntireXRT, 'input': input},
            response_cls=GetStyleSheetResponse,
        )


class FullTextSearchService(TcService):

    @classmethod
    def updateIndexIslandStatus(cls, applicationID: str, indexIslandUIDs: List[str], exportDate: datetime, status: int) -> ServiceData:
        """
        This operation updates the data islands that match the given input criteria with a status (1 - Replication
        Pending, 2 - Replication Complete, etc.) and exported date. Criteria include application ID, timestamp, and
        list of index island UIDs. 
        
        Use cases:
        Full Text Indexer error handling
        
        While indexing of data there might be issues during any step of the ETL (Extract, Transform and Load) process. 
        This operation will be used to set the status of the exported objects. The status that is updated will help in
        identifying the failed objects for error recovery. This operation will be used during indexing and sync when an
        error occurs.
        """
        return cls.execute_soa_method(
            method_name='updateIndexIslandStatus',
            library='Internal-AWS2',
            service_date='2015_03',
            service_name='FullTextSearch',
            params={'applicationID': applicationID, 'indexIslandUIDs': indexIslandUIDs, 'exportDate': exportDate, 'status': status},
            response_cls=ServiceData,
        )

    @classmethod
    def cleanupScratchTable(cls) -> ServiceData:
        """
        The scratch table contains records for the new and deleted objects. The applications consuming this data will
        have their unique application ID and the last processed timestamp entries in subscription table.  This
        operation deletes the records from the scratch table that are consumed by all the registered applications. The
        scratch table records whose last saved date is less than the minimum of the last processed date of subscription
        table will be deleted.
        
        Use cases:
        Full Text Indexer Sync
        
        When objects are created or deleted, the database triggers will add them to the scratch table.  During the Full
        Text Indexer Sync process, these new and deleted objects are queried and updated to the search index. On
        success, the data needs to be removed from the scratch table in order to avoid the table growing in size and
        leading to poor performance of queries. This new operation will help in cleaning only the data that is consumed
        by all the registered applications.  This operation will be called at the end or the sync process.
        """
        return cls.execute_soa_method(
            method_name='cleanupScratchTable',
            library='Internal-AWS2',
            service_date='2015_03',
            service_name='FullTextSearch',
            params={},
            response_cls=ServiceData,
        )

    @classmethod
    def deleteIndexedIslands(cls, applicationID: str, indexIslandUIDs: List[str]) -> ServiceData:
        """
        The accountability table contains records for the indexed objects. When the principal objects are deleted, the
        index is cleared but the records in the accountability table still exist. This operation deletes the records
        from the accountability table whose island UIDs and the application id match the given criteria.
        
        Use cases:
        Full Text Indexer Sync
        
        When objects are created or deleted, the database triggers will add them to the scratch table.  During the Full
        Text Indexer Sync process, it queries for the deleted objects and updates the search index. On success, the
        data needs to be removed from the accountability table in order to avoid the table growing in size and leading
        to poor performance of queries. This new operation will help in cleaning the data. This operation will be
        called during the delete process of sync.
        """
        return cls.execute_soa_method(
            method_name='deleteIndexedIslands',
            library='Internal-AWS2',
            service_date='2015_03',
            service_name='FullTextSearch',
            params={'applicationID': applicationID, 'indexIslandUIDs': indexIslandUIDs},
            response_cls=ServiceData,
        )

    @classmethod
    def deregisterApplicationIDs(cls, applicationIDs: List[str]) -> ServiceData:
        """
        This operation deletes the records of the given list of application IDs from the subscription table.
        
        Use cases:
        Full Text Indexer Sync
        
        The Initial Condition for this use case is that all the data from Teamcenter database has already been indexed
        in the Search Engine. When there are Access Manager rule tree changes and the change has a global impact, then
        all of the indexed data has to be refreshed. In order to identify the globally impacted objects, an application
        ID called :FTS_REFRESH (a unique identifier representing the Full Text Search Refresh operation) is registered
        with TIE (Teamcenter Import Export). Once the indexing is complete and there are no more objects left to be
        refreshed, this operation is invoked to deregister and to signal to TIE that the necessary cleanup of data in
        the database can be done. This is necessary to limit the growth of the database tables used by TIE.
        """
        return cls.execute_soa_method(
            method_name='deregisterApplicationIDs',
            library='Internal-AWS2',
            service_date='2015_03',
            service_name='FullTextSearch',
            params={'applicationIDs': applicationIDs},
            response_cls=ServiceData,
        )

    @classmethod
    def getAMImpactedObjects(cls, amSyncAppID: str, applicationID: str) -> GetAMImpactedObjectsResponse:
        """
        This operation lists the business objects whose READ access is impacted by the changes in Access Manager rule
        tree. The rule tree changes considered are limited to those made after the previous call to this operation.
        This operation is usually called periodically and the objects whose read access is modified due to the changes
        to Access Manager rule tree between the previous call and the current one are determined and returned.
        Optionally, this operation returns the set of objects which is the intersection of objects impacted by Access
        Manager rule changes and objects previously indexed. Previously indexed objects are stored in ACCT_TABLE table.
        The application id will be used to further filter the objects impacted by Access manager rule changes and got
        previously indexed for  a particular application.
        """
        return cls.execute_soa_method(
            method_name='getAMImpactedObjects',
            library='Internal-AWS2',
            service_date='2015_03',
            service_name='FullTextSearch',
            params={'amSyncAppID': amSyncAppID, 'applicationID': applicationID},
            response_cls=GetAMImpactedObjectsResponse,
        )

    @classmethod
    def getIndexedObjects(cls, applicationID: str, subscriptionAppID: str, exportedDate: datetime, status: int, classNames: List[str], maxObjectCount: int) -> GetIndexedObjectsResponse:
        """
        This operation queries for the indexed objects matching the given input criteria. Criteria include application
        ID, timestamp, export or import status of the object and list of object class names. 
        """
        return cls.execute_soa_method(
            method_name='getIndexedObjects',
            library='Internal-AWS2',
            service_date='2015_03',
            service_name='FullTextSearch',
            params={'applicationID': applicationID, 'subscriptionAppID': subscriptionAppID, 'exportedDate': exportedDate, 'status': status, 'classNames': classNames, 'maxObjectCount': maxObjectCount},
            response_cls=GetIndexedObjectsResponse,
        )

    @classmethod
    def getModifiedObjectsToSync(cls, applicationID: str) -> GetModifiedObjectsToSyncResponse:
        """
        This operation finds all indexed objects that have been modified since the last "sync" run. 
        """
        return cls.execute_soa_method(
            method_name='getModifiedObjectsToSync',
            library='Internal-AWS2',
            service_date='2015_03',
            service_name='FullTextSearch',
            params={'applicationID': applicationID},
            response_cls=GetModifiedObjectsToSyncResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class GenerateMonolithicJtOptions(TcBaseObj):
    """
    Configuration structure.
    
    :var continueOnError: if 0 return on any error condition raised, 1 indicates to continue
    :var processIntermediateNodes: 0-no 1-replace 2-add
    :var remoteProcessing: false mean call API without TSTK, true-means use TSTK and return
    after waiting for number of seconds sepcified by waitSecondsAtServer for the translation to complete.
    In this case client must use EndGenerateMonolithicJt to see the status.
    :var waitSecondsAtServer: Used in case of TSTK.
    """
    continueOnError: int = 0
    processIntermediateNodes: int = 0
    remoteProcessing: bool = False
    waitSecondsAtServer: int = 0


@dataclass
class BeginGenerateMonolithicJtResponse(TcBaseObj):
    """
    BeginGenerateMonolithicJtResponse struct
    
    :var ticket: The transient file ticket to be used for downloading the generated monolithic jt file
    :var logFileticket: The transient file ticket to be used for downloading the generated log file
    :var request: If using TSTK, the Request object created for translation purposes
    :var currentState: If using TSTK the current state of the translation
    :var data: partial failures are returned - along with object ids for each plmxml data could not be generated.
    """
    ticket: str = ''
    logFileticket: str = ''
    request: BusinessObject = None
    currentState: str = ''
    data: ServiceData = None


@dataclass
class EndGenerateMonolithicJtResponse(TcBaseObj):
    """
    EndGenerateMonolithicJtResponse struct
    
    :var ticket: The transient file ticket to be used for downloading the generated monolithic jt file
    :var logFileticket: The transient file ticket to be used for downloading the generated log file
    :var currentState: If using TSTK the current state of the translation. Completed to mean - that the
    translation is done.
    :var data: data
    """
    ticket: str = ''
    logFileticket: str = ''
    currentState: str = ''
    data: ServiceData = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Fnd0SecondaryAudit, Fnd0AuditLog
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetProcessHistoryAuditRecordsResponse(TcBaseObj):
    """
    Contains a list of  ProcessHistoryAuditRecordStruct structures (which contain process histroy audit record entry,
    chilren process histroy audit entries, audit record entries of secondary obeject associated with the process ) as
    well as a list of process names. Retrieved audit record entries will be added into standard ServiceData list of
    plain. Any failure will be returned in the ServiceData list of partial errors.
    
    :var processNames: Process names of multi process the given workspace object goes through.
    :var rootProcessHistoryAuditRecords: Root process histrory audit record entries for each process.
    :var serviceData: The service data.
    """
    processNames: List[str] = ()
    rootProcessHistoryAuditRecords: List[ProcessHistoryAuditRecordStruct] = ()
    serviceData: ServiceData = None


@dataclass
class ProcessHistoryAuditRecordStruct(TcBaseObj):
    """
    Contains one process histroy audit record entry and its children's audit records as well as secondary audit records
    of attachments/targets associated with it.
    
    :var auditRecord: One process history audit record entry of given workspace object.
    :var childrenAuditRecords: Children process history audit records of auditRecord.
    :var secondaryAuditRecords: Audit records of secondary obejct such as attachements/targets assoicated with the
    audit record (auditRecord)  of certain process.
    """
    auditRecord: Fnd0AuditLog = None
    childrenAuditRecords: List[ProcessHistoryAuditRecordStruct] = ()
    secondaryAuditRecords: List[Fnd0SecondaryAudit] = ()



from tcsoa.gen.ConfigFilterCriteria._2011_06.services import EffectivityManagementService as imp0
from tcsoa.gen.ConfigFilterCriteria._2013_05.services import EffectivityManagementService as imp1
from tcsoa.gen.ConfigFilterCriteria._2017_05.services import EffectivityManagementService as imp2
from tcsoa.gen.ConfigFilterCriteria._2013_05.services import VariantManagementService as imp3
from tcsoa.base import TcService


class EffectivityManagementService(TcService):
    convertEffectivityExpressions = imp0.convertEffectivityExpressions
    convertEffectivityTables = imp0.convertEffectivityTables
    getAvailableProductEffectivity = imp0.getAvailableProductEffectivity
    getConfigurableProducts = imp0.getConfigurableProducts
    getEffectivityConditions = imp0.getEffectivityConditions
    getEffectivityDisplayString = imp1.getEffectivityDisplayString
    getEffectivityExpressions = imp0.getEffectivityExpressions
    getEffectivityOverlapStates = imp1.getEffectivityOverlapStates
    getEffectivitySubsetTables = imp2.getEffectivitySubsetTables
    getEffectivityTables = imp0.getEffectivityTables
    getRevRuleEffectivityCriteria = imp0.getRevRuleEffectivityCriteria
    setEffectivityConditionSubsets = imp2.setEffectivityConditionSubsets
    setEffectivityConditions = imp0.setEffectivityConditions
    setRevRuleEffectivityCriteria = imp0.setRevRuleEffectivityCriteria


class VariantManagementService(TcService):
    getProductConfigurations = imp3.getProductConfigurations



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class ReportsCriteria2(TcBaseObj):
    """
    Criteria needed to retrieve report definitions.
    At least one of the optional parameters must be included.
    
    :var clientId: Client identifier for routing purposes (required)
    :var reportDefinitionId: Report definition ID (optional)
    :var reportDefinitionName: Report definition name (optional)
    :var category: Designates report category, e.g. item, summary or custom (optional)
    :var source: Solution source of report definition, e.g. Teamcenter, TcRA (optional)
    :var status: For future use, may be null
    :var contextObjects: A vector of ID's representing context object(s) (required for item reports).
    """
    clientId: str = ''
    reportDefinitionId: str = ''
    reportDefinitionName: str = ''
    category: str = ''
    source: str = ''
    status: str = ''
    contextObjects: List[BusinessObject] = ()



from __future__ import annotations

from typing import List
from tcsoa.gen.Internal.Integration._2008_06.IntegrationManagement import RenameIMFInfo
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class IntegrationManagementService(TcService):

    @classmethod
    def renameIMF(cls, infos: List[RenameIMFInfo]) -> ServiceData:
        """
        Sets the original file name on an input file object.
        """
        return cls.execute_soa_method(
            method_name='renameIMF',
            library='Internal-Integration',
            service_date='2008_06',
            service_name='IntegrationManagement',
            params={'infos': infos},
            response_cls=ServiceData,
        )



from __future__ import annotations

from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from tcsoa.gen.BusinessObjects import UserSession
from dataclasses import dataclass


@dataclass
class GetAvailableWorkspacesResponse(TcBaseObj):
    """
    Response of getAvailableWorkspaces operation.
    
    :var workspacesViewModel: The list of workspace view model objects in JSON format which can be bound to declarative
    view.
    :var serviceData: The service data object.
    """
    workspacesViewModel: str = ''
    serviceData: ServiceData = None


@dataclass
class GetTableViewModelPropertiesIn(TcBaseObj):
    """
    Input for getTableViewModelProperties
    
    :var objectUids: The list of the object UIDs for which the Table ViewModel properties have to be fetched.
    :var requestPreference: Key value pair (string, list of string) passed to the GetTableViewModelPropRequestPref
    operation to control the behavior. Supported keys:
    1. "typesToInclude" : Internal type names  to be included along with types discovered during the operation.
    
    2. "supportsCardinality" value could be "true" or "false".
    :var columnConfigInput: Input to fetch the column configuration.
    """
    objectUids: List[str] = ()
    requestPreference: GetTableViewModelPropRequestPref = None
    columnConfigInput: ColumnConfigInput = None


@dataclass
class GetTableViewModelPropsOut(TcBaseObj):
    """
    Output containing the JSON string representation for the objects for which the properties were fetched and the
    column configuration data.
    
    :var viewModelPropertiesJsonString: JSON string that contains the view model properties, the schema of JSON string
    is defined in getDeclarativeStyleSheet2 service operation.
    :var columnConfig: The column configuration data.
    """
    viewModelPropertiesJsonString: str = ''
    columnConfig: ColumnConfig = None


@dataclass
class GetTableViewModelPropsResp(TcBaseObj):
    """
    GetTableViewModelProperties response
    
    :var output: The model objects with the dynamic compound properties inflated and the column configuration
    applicable for the input objects.
    :var serviceData: Service Data
    """
    output: GetTableViewModelPropsOut = None
    serviceData: ServiceData = None


@dataclass
class GetViewModelProps2Response(TcBaseObj):
    """
    Objects for which properties could be successfully retrieved are returned as JSON string. Any failures will be
    returned with the input object mapped to the error message in the ServiceData list of partial errors.
    
    :var viewModelPropertiesJsonString: JsonSON string that contains the view model properties, the schema of JSON
    string is defined in getDeclarativeStyleSheet2 service operation.
    :var serviceData: Service data
    """
    viewModelPropertiesJsonString: str = ''
    serviceData: ServiceData = None


@dataclass
class LoadViewModelForEditing2Response(TcBaseObj):
    """
    Response of LoadViewModelForEditing2 operation
    
    :var viewModelObjectsJsonStrings: Json strings that contain the ViewModelObject structures which provide
    information about the editable status of the properties passed in the input. This information can be used by client
    to show which properties as editable. The ViewModelObjectJsonString is also used in getDeclarativeStyleSheet2
    operation
    :var serviceData: Partial errors are returned in the Service Data. The following partial errors may be returned:
    302021: The input object is not modifiable.
    302022: The property is not modifiable. 
    302024. The traversal path of dynamic compound property has changed
    """
    viewModelObjectsJsonStrings: List[str] = ()
    serviceData: ServiceData = None


@dataclass
class ColumnConfig(TcBaseObj):
    """
    The column configuration data.
    
    :var columnConfigId: Column configuration ID.
    :var operationType: Operation type used for determining columns. Supported values are: "Union" and "Intersection".
    :var columns: List of available columns.
    :var typesForArrange: The valid Teamcenter types used for fetching the columns.
    """
    columnConfigId: str = ''
    operationType: str = ''
    columns: List[ColumnDefInfo] = ()
    typesForArrange: List[str] = ()


@dataclass
class TCSessionAndAnalyticsInfo(TcBaseObj):
    """
    - userSession The user session object
    - extraInfoOut Map of key/value pairs (string/string). Some/all/none of the following keys and values are returned,
    depending on what was passed in extraInfoIn:
    - TCServerVersion The version of the Teamcenter server.
    - hasProjects "true" or "false" depending on whether the user has projects
    - AWC_StartupPreferences This contains list of preferences to be retrieved at startup by the Active workspace
    client from the Teamcenter server.
    - AWC_Startuptypes List of types to be loaded at start up by the Active workspace client from the Teamcenter server.
    - DefaultDateFormat The default date format required.
    - AWServerVersion The version of the Active workspace
    - typeCacheLMD UTC formatted last modified date for Type cache Dataset.
    - WorkspaceId Workspace Id for current Active workspace user session.
    - AWC_PostLoginStages This preference represents list of post login stages in the sequence to be displayed in the
    Active worksapace after successful authentication.
    - analyticsData This contains an instance of AnalyticsData required to hold software analytics data for logged in
    user. 
    - serviceData The serviceData.
    
    
    
    :var userSession: The user session object
    :var extraInfoOut: extraInfoOut Map of key/value pairs (string/string). Some/all/none of the following keys and
    values are returned, depending on what was passed in extraInfoIn:
    - TCServerVersion The version of the Teamcenter server.
    - hasProjects "true" or "false" depending on whether the user has projects
    - AWC_StartupPreferences This contains list of preferences to be retrieved at startup by the Active workspace
    client from the Teamcenter server.
    - AWC_Startuptypes List of types to be loaded at start up by the Active workspace client from the Teamcenter server.
    - DefaultDateFormat The default date format required.
    - AWServerVersion The version of the Active workspace
    - typeCacheLMD UTC formatted last modified date for Type cache Dataset.
    - WorkspaceId Workspace Id for current Active workspace user session.
    - AWC_PostLoginStages This preference represents list of post login stages in the sequence to be displayed in the
    Active worksapace after successful authentication.
    
    
    :var analyticsData: This contains an instance of AnalyticsData required to hold software analytics data for logged
    in user.
    :var serviceData: The serviceData.
    """
    userSession: UserSession = None
    extraInfoOut: SessionExtraInfo = None
    analyticsData: AnalyticsData = None
    serviceData: ServiceData = None


@dataclass
class ColumnConfigInput(TcBaseObj):
    """
    Contains input information required to retrieve UI column configurations from the Teamcenter database.
    
    :var clientName: The name of a client application, as represented by an instance of Fnd0Client in the Teamcenter
    database. This value must match the value of fnd0ClientName property. 
    For example: The client name for Active Workspace is "AWClient".
    :var hostingClientName: Specifies the name of a hosting client application, as represented by an instance of
    Fnd0Client, in the Teamcenter databases. This value must match a value of the fnd0ClientName property.
    
    For example: If client A is integrated with client B and the user can invoke client B commands from within client
    A, the input to getUiConfigs would specify client A as hosting client and client B as the client. If the caller
    wanted native commands for client A, client A would be specified as client and hosting client would be empty.
    :var clientScopeURI: The unique name of the client scope containing column configurations.
    For example: "Awp0SearchResults" is the client scope URI for search location.
    :var operationType: The operation that needs to be applied to finalize the columns to be returned back.
    Valid values are:
    "Intersection" - Gets the intersection of the columns for the types found in search results.
    "Union" - Gets all the columns for the types found in search results.
    "Configured" - Gets all the columns defined for requested scope irrespective of types passed in. If it does not
    find any configuration at the specified scope it will search up in the order of scopes User, Role, Group and Site.
    No need to pass type names if need to fetch all columns.
    :var columnsToExclude: List of columns which should be excluded from the final list being returned. The value
    provided should be in the format "TypeName.PropertyName". Both type name and property name should be internal
    values. 
    For example: ItemRevision.sequence_id, where '.' is the delimiter.
    """
    clientName: str = ''
    hostingClientName: str = ''
    clientScopeURI: str = ''
    operationType: str = ''
    columnsToExclude: List[str] = ()


@dataclass
class ColumnDefInfo(TcBaseObj):
    """
    Column Definition
    
    :var displayName: The display name for the value displayed in the column header.
    :var associatedTypeName: The business object type for the value displayed in the column. This can be any valid
    Teamcenter business object type.
    :var propertyName: The property name for the value displayed in the column.
    :var pixelWidth: The pixel width for the column. Valid pixel widths are integer values between 1 and 500.
    :var columnOrder: The column order value is used to arrange the columns in order.
    :var hiddenFlag: If true, the column should be hidden on the client user interface.
    :var sortPriority: Sort priority set on column helps identify the order in which the columns should be used during
    sort.
    Sort priority value will be zero for columns not marked for sorting.
    :var sortDirection: How the columns are sorted. Supported values are: "Ascending" and "Descending". This value will
    be empty if the column is not marked for sorting.
    """
    displayName: str = ''
    associatedTypeName: str = ''
    propertyName: str = ''
    pixelWidth: int = 0
    columnOrder: int = 0
    hiddenFlag: bool = False
    sortPriority: int = 0
    sortDirection: str = ''


@dataclass
class AnalyticsData(TcBaseObj):
    """
    This contains an instance of AnalyticsData required to hold software analytics data for logged in user.
    
    :var isDataCollectionEnabled: This value for preference TC_ProductExcellenceProgram to indicate the software
    analytics should be enabled or not.  If data collection is off then the server will not send any other information
    and an instance of AnalyticsData will be empty, this value is set to false.
    :var useInternalServer: Where the analytice data is logged:. If true, send analytics data to analytics internal
    server; otherwise, send analytics data to analytics cloud based server (AWS).
    :var burstTimeInterval: The time interval at which software analytics data will be sent to the SA server. This
    value is a recommendation since all clients cannot implement a burst interval other than zero.
    :var analyticsExtraInfo: Map of key\value pairs (string, string). These are optional fields sent by the server and
    will be sent to the analytics site as is. The intent is that these fields, if present, will be sent to the SA
    server retaining their name and value.
    """
    isDataCollectionEnabled: bool = False
    useInternalServer: bool = False
    burstTimeInterval: int = 0
    analyticsExtraInfo: AnalyticsExtraInfo = None


"""
Map of key\value pairs (string, string). These are optional fields returned to the client for the purpose of forwarding to the analytics site. There is no guarantee as to the contents or ordering of this map. The contents may vary across platforms or Teamcenter versions. The values in the map should NOT be interpreted by the client or used for client side logic since there is no guarantee on the content.
"""
AnalyticsExtraInfo = Dict[str, str]


"""
The map of (string, string) filter parameters. 
Following are the possible values.
1. Key "scope", Value could be "session" or "all". If the value is passed as "session", only the workspaces which match other filter parameters and apply to current user session (group role combination) will be returned. If it is passed as "all", all the workspaces in the system matching other filters will be returned regardless of existing user session.
2. Key "group", Value needs to be the group ID to which associated workspaces needs to be loaded. Key "group" is optional. If it is not present in operation input, all the workspaces matching other filters will be returned. 
3. Key "role", Value needs to be the role ID to which associated workspaces needs to be loaded. Key "role" is optional. If it is not present in operation input, all the workspaces matching other filters will be returned.
"""
FilterParams = Dict[str, str]


"""
Key value pair passed to the getTableViewModelProperties operation to control the behavior.
"""
GetTableViewModelPropRequestPref = Dict[str, List[str]]


"""
Map of key-value pairs. Returned value depends on what was passed in extraInfoIn:
TCServerVersion The version of the Teamcenter server.
hasProjects true or false depending on whether the user has projects
"""
SessionExtraInfo = Dict[str, str]


"""
String map
"""
StringMap8 = Dict[str, str]



from tcsoa.gen.Internal_ProjectManagementAw._2019_12.services import ScheduleManagementAwService as imp0
from tcsoa.base import TcService


class ScheduleManagementAwService(TcService):
    manageProgramView = imp0.manageProgramView



from __future__ import annotations

from typing import List
from tcsoa.gen.BusinessObjects import POM_object
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class DeleteIn(TcBaseObj):
    """
    The DeleteIn structure is used to hold the input container, objects to be deleted associated as the property of the
    container. This structure also holds a flag to unlink the objects from the input container in case deletion fails.
    
    :var container: A  POM_object that is related to the objects to be deleted.
    :var objectsToDelete: A list POM_object objects be unrelated from the container object and  then deleted.
    :var property: The  name of the TypedRef or relation property with which the objects to be deleted are related to
    the input container
    :var unlinkAlways: If true the operation will unlink the objects from the input container in case of delete
    operation otherwise they remained linked.
    """
    container: POM_object = None
    objectsToDelete: List[POM_object] = ()
    property: str = ''
    unlinkAlways: bool = False



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, WorkspaceObject, VariantRule, Cfg0ConfiguratorPerspective
from typing import Dict, List
from tcsoa.gen.Internal.ProductConfiguratorAw._2017_12.ConfiguratorManagement import OptionValueSelection, OptionFamily
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class OptionFamily(TcBaseObj):
    """
    A collection of features. It also contains selection state and allowed selection states for the current rendered
    family.
    
    :var selectionInfo: This will hold the nodeID of the Feature and its selection state.
    :var familyObj: ViewModelObject which represents family object information e.g. displayName, type.
    :var features: A list of all allowable selection states for option family based on current option value selections
    and constraint rules. Valid values refer to 1.1.1.10 section.
    :var allowedSelectionStates: A list of all allowable selection states for option value based on current option
    value selections and constraint rules. Following are the valid values:
    
    NoSelection ->0
    User positive Selection->1
    User negative Selection->2
    systemDefault positive Selection->5
    systemDefault negative Selection-> 6
    system positive Selection -> 9
    system negative Selection -> 10
    :var labelInfo: A map (string, vector<string>) of label type and list of nodeID of ViewModelObject.
    Valid keys : "violations", "package"
     Values: nodeID of ViewModelObject
     If key ==  "violation", then:
     Value = list of appropriate nodeID of a ViewModelObject of type "violation".
    """
    selectionInfo: Selection = None
    familyObj: ViewModelObject = None
    features: List[Feature] = ()
    allowedSelectionStates: List[int] = ()
    labelInfo: StringMap = None


@dataclass
class Scope(TcBaseObj):
    """
     Scope stores the collection of variant option families. Scope represents the top level object in the hierarchy of
    configuration data that is displayed in the client.
    
    :var nodeID: The UID of the variant option group or partition.
    :var scopeObj: This ViewModelObject store information of scope object such as nodeID, displayName, internalName.
    :var families: A list of option families within this group.
    """
    nodeID: str = ''
    scopeObj: ViewModelObject = None
    families: List[OptionFamily] = ()


@dataclass
class SelectedExpression(TcBaseObj):
    """
    Structure containing expression selection for the given affected object. If the expression has split then there
    will be more than one SelectedExpression structure with same affectedObject UID.
    
    :var affectedObject: UID of the Affected Object.
    :var displayValue: Display value of the affected object.
    :var userSelections: Represents list of Map (string, OptionValueSelection) of all option values selected by user
    per family.
    :var familySelections: Represents list of Map (string, int) of all families selected by user. Key is UID of
    Cfg0AbsFamily object and value is selection state of the family. 
     0 &ndash; Unselected.
     1 - User selected (equal to).
     2 - Negatively user selected (not equal to).
     3 - System Selected (equal to).
     4 - Negatively system selected (not equal to).
     5 - Default Selected (equal to).
     6 - Negatively default selected (not equal to).
    """
    affectedObject: str = ''
    displayValue: str = ''
    userSelections: UserSelectionMap2 = None
    familySelections: FamilySelectionMap = None


@dataclass
class SelectedExpressions(TcBaseObj):
    """
    Store selections against affected object UID.
    
    :var affectedObject: Object UID for which selection is stored, valid values are UIDs of Awb0Element, VariantRule,
    and Mdl0ModelElement.
    :var columnId: For split cases it will hold the column number for given affected object.
    :var selections: Map of (OptionFamily nodeID and list of Selections) nodeID of OptionFamily or Feature and list of
    Selection.
    """
    affectedObject: str = ''
    columnId: int = 0
    selections: SelectionMap = None


@dataclass
class Selection(TcBaseObj):
    """
    This structure stores nodeID of OptionFamily or Feature along with its selection state.
    
    :var nodeID: The nodeID of the OptionFamily or Feature structures in the response.
    :var selectionState: The state of object selection. The below are given valid values.
    Selection state values are
    NoSelection ->0
    User positive Selection->1
    User negative Selection->2
    systemDefault positive Selection->5
    systemDefault negative Selection-> 6
    system positive Selection -> 9
    system negative Selection -> 10
    """
    nodeID: str = ''
    selectionState: int = 0


@dataclass
class SelectionSummary(TcBaseObj):
    """
    Pair of option family name to selected option value.
    
    :var selectionState: The state of Cfg0AbsValue or Cfg0AbsFamily selection.
    :var familyDisplayName: Display name of the option family (Cfg0AbsFamily) selected. In case of Boolean or multi
    select families, this parameter would contain the option value display name.
    :var valueDisplayName: Display name of the option value (Cfg0AbsValue) selected. In case of Boolean, multi select
    families or family selection, this parameter would be empty.
    """
    selectionState: int = 0
    familyDisplayName: str = ''
    valueDisplayName: str = ''


@dataclass
class SelectionsSummary(TcBaseObj):
    """
    Summary of all selections in displayable format.
    
    :var summaryOfSelections: A list of all selections per family in user friendly displayable format.
    :var requiredFamilies: Information about which families are mandatory.
    :var displayName: Display name of selections summary panel.
    :var internalName: Internal name of selections summary panel. This name is used to identify if user has requested
    to view selections summary panel.
    """
    summaryOfSelections: List[SelectionSummary] = ()
    requiredFamilies: List[SelectionSummary] = ()
    displayName: str = ''
    internalName: str = ''


@dataclass
class ValidateProductConfigResponse(TcBaseObj):
    """
    It is the structure containing response of operation validateProductConfiuration.
    
    :var validateProductConfigurationOutputs: A list of ValidateProductConfigurationOutput structure. The sequence in
    this list is same as the input sequence of SelectedExpression.
    :var serviceData: ServiceData containing partial exceptions, if any. Partial exceptions are added as per index of
    input SelectedExpression in the list.
    """
    validateProductConfigurationOutputs: List[ValidateProductConfigurationOutput] = ()
    serviceData: ServiceData = None


@dataclass
class ValidateProductConfigurationOutput(TcBaseObj):
    """
    It is the structure containing the violations and SelectionMap for each affected object.
    
    :var criteriaStatus: Indicate if input configuration is valid or not. If true, input configuration is valid;
    otherwise, input configuration is invalid.
    :var valueToViolations: A map (string, Violations) of all violated values.
    :var outputSelections: A map (string, list of OptionValueSelection) of expanded selections that is in accordance
    with the given input selections and the rules in the context.
    """
    criteriaStatus: bool = False
    valueToViolations: ValueToViolations = None
    outputSelections: UserSelectionMap2 = None


@dataclass
class VariantConfigurationViewIn(TcBaseObj):
    """
    The input containing the selected context and selections to fetch the available variability.
    
    :var configPerspective: The configurator perspective. When this operation is invoked first time, this parameter
    would be null. But in all subsequent calls, this parameter must be populated with the configurator perspective
    retrieved in the response.
    :var selectedContext: The selected object for which variant configurator context to be retrieved. It can be a
    runtime business object (instance of Awb0Element) or persistent variant configurable business object (instance of
    Mdl0ConditionalElement). This parameter is ignored if configPerspective is passed in input.
    :var activeVariantRules: An optional list of active VariantRule whose expression can be used as input selections
    for computing next valid selections. The parameter is ignored if the &lsquo;selections&rsquo; is populated.
    :var scopes: A list of UIDs of Scope (Cfg0AbsFamilyGroup, Ptn0Partition) for which the next valid selections must
    to be retrieved. If this parameter is empty, then option values from model group (Cfg0AbsFamilyGroup) are are
    retrieved.
    :var selections: List of all user and system selections for the custom configuration or the configurable object.
    :var payloadStrings: A map (string, list of strings) represents the payload that is transferred between client and
    server for the current session. The operation would use the payload as the state information to process the request.
    The client should pass the same payloadStrings that is returned by previous call if the state information is to be
    maintained for the current request.
    Valid map:
    "violation" -> { "list of nodeIDs of viewModelObject which hold the violation" }
    :var requestInfo: Map (string, string) of request names and value pairs. Allowed names are: 
    -"requestType" : This gives the operation to be handled for the SOA call.   
    [ "requestType" -> { ("viewPackage") | ("selectPackage") | ("addPackage") | ("cancelPackage") | ("getConfig") |
    ("validate") | ("switchToManual") | ("expand") }]
    - "selectedPackage" : nodeID of selected package feature object. This is required in case of requestType is one of
    "viewPackage", "selectPackage", "cancelPackage", "addPackage".
    - "configurationControlMode" is the configuration view mode in manual mode invalid selections are allowed.
    ["configurationControlMode" -> { "guided" | "manual" }]
    - "severity" : This will give information what all violations you want to show.
    ["severity" &#61672; { "error" | "warn" | "info" }]
    -"viewName" : This will hold the current configuration view name.
    [" viewName" &#61672; { "guidedConfig" | "manualWithError" | "manualWithExpand" | "package"}]
    """
    configPerspective: Cfg0ConfiguratorPerspective = None
    selectedContext: BusinessObject = None
    activeVariantRules: List[VariantRule] = ()
    scopes: List[str] = ()
    selections: List[SelectedExpressions] = ()
    payloadStrings: StringMap = None
    requestInfo: StringMap = None


@dataclass
class VariantConfigurationViewResponse(TcBaseObj):
    """
    The output containing the available variability for the input list of variant option value selections.
    
    :var configPerspective: The configurator perspective containing all information about the current Configurator
    Context, revision rule and effectivity. All further communications with the server to retrieve variant
    configuration data must use this object.
    :var scopes: UID of the current expanded group or partition.
    :var allScopes: The list of all variant option groups available in the current Configurator Context.
    :var selections: A list of SelectedExpressions strutures.
    :var selectionsSummary: The summary of all selections in displayable format.
    :var labels: ViewModelObjectLabelMap of labels.
    :var payloadStrings: A map (string, list of strings) represents the payload that is transferred between client and
    server for the current session. The operation would use the payload as the state information to process the request.
    :var responseInfo: Map (string, string) of response names and value pairs. Allowed names are:
    -"requestType" : This gives the operation to be handled for the SOA call.   
    [ "requestType" -> { ("viewPackage") | ("selectPackage") | ("addPackage") | ("cancelPackage") | ("getConfig") |
    ("validate") | ("switchToManual") | ("expand") }]
    - "selectedPackage" : nodeID of selected package feature object. This is required in case of requestType is one of
    "viewPackage", "selectPackage", "cancelPackage", "addPackage".
    - "configurationControlMode" is the configuration view mode in manual mode invalid selections are allowed.
    ["configurationControlMode" -> { "guided" | "manual" }]
    - "severity" : This will give information what all violations you want to show.
    ["severity" &#61672; { "error" | "warn" | "info" }]
    :var serviceData: Contains the list of all BusinessObjects that make up the output, as well as any errors that
    might have occurred as part of the service invocation.
    """
    configPerspective: Cfg0ConfiguratorPerspective = None
    scopes: List[str] = ()
    allScopes: List[Scope] = ()
    selections: List[SelectedExpressions] = ()
    selectionsSummary: SelectionsSummary = None
    labels: ViewModelObjectLabelMap = None
    payloadStrings: StringMap = None
    responseInfo: StringMap = None
    serviceData: ServiceData = None


@dataclass
class VariantExpressionDataInput(TcBaseObj):
    """
    The input structure for the getVariantExpressionData operation containing either the selected objects or current
    expanded families and optionally containing configurator context, configurator perspective, and variant expression
    filters.
    
    :var configContextProvider: The configurator context provider object. It is optional. If not null, it is used to
    find configurator context object. Valid input types are ItemRevision, Awb0Element and Mdl0ConditionalElement. This
    parameter is ignored if configContext or configPerspective parameters are passed.
    :var configContext: The variant configurator context. Valid input types are Item or ItemRevision. This parameter is
    ignored if the configPerspective parameter is passed.
    :var configPerspective: Input Cfg0ConfiguratorPerspective. It is optional. Cfg0ConfiguratorPerspective object is
    used as communication mechanism for the configuration and the valid configurator contexts across the Product
    Configurator and its consuming applications such as 4G Designer, Structure Manager. If it is empty then the config
    perspective is obtained from the selected objects.
    :var selectedObjects: The list of selected business objects. If this parameter is empty then
    currentExpandedFamilies and configPerspective should be provided. The supported business object types are
    Awb0Element, Mdl0ConditionalElement and variant configurable business objects - BOMLine, VariantRule.
    :var currentExpandedFamilies: The list of expanded option families. If the input contains only
    currentExpandedFamilies and configPerspective as non-empty input then operation will return only the Option values
    of the provided families.
    :var filters: The input filters that govern the option family and value response. For instance input filter may
    contain option filter (&lsquo;showCurrentElements&rsquo;, &lsquo;showFamilies&rsquo; ) or intent filters (from
    Cfg0ObjectIntentions List Of Values).
    """
    configContextProvider: BusinessObject = None
    configContext: WorkspaceObject = None
    configPerspective: Cfg0ConfiguratorPerspective = None
    selectedObjects: List[BusinessObject] = ()
    currentExpandedFamilies: List[str] = ()
    filters: VariantExpressionFilters = None


@dataclass
class VariantExpressionDataResponse(TcBaseObj):
    """
    Variant expression response for the selected objects.
    
    :var optionFamilies: List of option Families.
    :var selectedExpression: The list of selections of all option values that form the expression on the input object.
    If the expression has split then there will be more than one SelectedExpression structure with same affectedObject
    UID.
    :var configPerspective: Output configurator perspective. Configurator Perspective object is used as communication
    mechanism for the configuration and the valid configurator contexts across the Product Configurator and its
    consuming applications such as 4G Designer, Structure Manager.
    :var filters: The filters applied on the output to render the option families and values.
    :var serviceData: Service Data containing partial errors, if any.
    """
    optionFamilies: List[OptionFamily] = ()
    selectedExpression: List[SelectedExpression] = ()
    configPerspective: Cfg0ConfiguratorPerspective = None
    filters: VariantExpressionFilters = None
    serviceData: ServiceData = None


@dataclass
class VariantExpressionFilters(TcBaseObj):
    """
    Variant expression filter structure containing intent filters and option filter.
    
    :var intentFilters: Represents the list of Intent filters. The valid values are the Cfg0ObjectIntentions List Of
    Values.
    :var optionFilter: Option output filter. Valid values are &lsquo;showCurrentElements&rsquo;,
    &lsquo;showFamilies&rsquo;.
    """
    intentFilters: List[str] = ()
    optionFilter: str = ''


@dataclass
class ViewModelObject(TcBaseObj):
    """
    The generic object that is used in this SOA request and response.
    User facing object that is used by the client of this SOA. It encapsulates a model object in server side.
    This object represent each object which is visible on UI like Scope, Feature, OptionFamily.
    
    :var wsObject: WorkspaceObject to be rendered.
    :var sourceUid: The UID of the object to be rendered. Valid values are either valid UIDs of configurable object
    along with context UIDs.
    :var displayName: The display name of the object.
    :var sourceType: Type of the ViewModelObject. Valid values are following Business object types: Cfg0AbsFamilyGroup,
    Cfg0PackageOptionValue, Cfg0AbsValue, Cfg0AbsFamily.
    :var props: Map of (string, list of strings) ViewModelObject property and value.
    Valid keys: "isFreeForm", "isMultiSelect", "isPackage".
    """
    wsObject: WorkspaceObject = None
    sourceUid: str = ''
    displayName: str = ''
    sourceType: str = ''
    props: StringMap = None


@dataclass
class ViewModelObjectLabelType(TcBaseObj):
    """
    ViewModelObjectLabelType will be used in ViewModelObjectLabelMap to get the violation and package information.
    
    :var nodeMap: Map of string, vector<ViewModelObject> ViewModelObject.
    """
    nodeMap: ViewModelObjectMap = None


@dataclass
class Violation(TcBaseObj):
    """
    This structure contains list of severities and messages for violations.
    
    :var severities: A list of severities of violations found during validation.
    :var messages: A list of messages of violations found during validation.
    """
    severities: List[int] = ()
    messages: List[str] = ()


@dataclass
class Feature(TcBaseObj):
    """
    Feature stores the information for the Variant feature to be displayed to the user. This contains the current
    selection state of the feature and labels such as violations.
    
    :var selectionInfo: This will hold the nodeID of the Feature and its selection state.
    :var featureObject: ViewModelObject which represents feature object information.
    :var allowedSelectionStates: A list of all allowable selection states for option value based on current option
    value selections and constraint rules. Following are the valid values:
    
    NoSelection ->0
    User positive Selection->1
    User negative Selection->2
    systemDefault positive Selection->5
    systemDefault negative Selection-> 6
    system positive Selection -> 9
    system negative Selection -> 10
    :var labelInfo: A map (string, list of strings) of label type and list of nodeID of ViewModelObjects.
    nodeIDViewModelObjectIf key == "violation", then:
       Value = list of appropriate nodeID of a ViewModelObjects of type "violation".
    """
    selectionInfo: Selection = None
    featureObject: ViewModelObject = None
    allowedSelectionStates: List[int] = ()
    labelInfo: StringMap = None


@dataclass
class FilterPanelDataResponse(TcBaseObj):
    """
    Filter Panel Data response for the selected object.
    
    :var featureParameters: The details of feature filter parameter Current Elements, Show Families and their selection
    state. Current Elements and Show Families are display name defined in text server locale file.
    :var intentParameters: The details of Intents filter parameter Manufacturing, Marketing, and Technical. These
    intents will be retrieve from LOV Cfg0ObjectIntentions.
    """
    featureParameters: List[FilterParameter] = ()
    intentParameters: List[FilterParameter] = ()


@dataclass
class FilterParameter(TcBaseObj):
    """
    Filter parameter details and its selection state.
    
    :var internalName: Internal name of the filter parameter.
    :var displayName: Display name of the filter parameter.
    :var isSelected: If true the parmeter is selected.
    """
    internalName: str = ''
    displayName: str = ''
    isSelected: bool = False


"""
Map(string, OptionValueSelection) of all option values selected by user per family.
"""
UserSelectionMap2 = Dict[str, List[OptionValueSelection]]


"""
A map (string, Violations) of all violated values.
"""
ValueToViolations = Dict[str, Violation]


"""
Map of string, and list of ViewModelObjectLabelType which will hold ViewModelObjects for violations and package.

The valid keys are "violations", "package"
The value is ViewModelObjectLabelType which contains the ViewModelObjects in a map where key names are nodeID and values are corresponding "violation" or "package" ViewModelObjects.
"""
ViewModelObjectLabelMap = Dict[str, List[ViewModelObjectLabelType]]


"""
Map of string and list of ViewModelObject. The key represents the nodeID of the ViewModelObject.
This map is useful for the client to obtain ViewModelObject from labels structure given a nodeID.
For example: The server will send the response of labelInfo Map in Feature structure containing violations in the following format, 
labelInfo["violations"] = [ "nodeID1", "nodeID3", "nodeID5" ]
The client will use the nodeIDs list and look into the ViewModelObjectMap of the "violations" label and fetch the corresponding ViewModelObject for the given nodeID.
"""
ViewModelObjectMap = Dict[str, List[ViewModelObject]]


"""
Map(string, int) of all family selected by user.
"""
FamilySelectionMap = Dict[str, int]


"""
Map of OptionFamily nodeID and list of Selections.
Map key will hold the nodeID of the OptionFamily and values contain the Selection list for features of the family.
"""
SelectionMap = Dict[str, List[Selection]]


"""
Map of string and list of strings.
"""
StringMap = Dict[str, List[str]]



from __future__ import annotations

from tcsoa.gen.Internal.TCXMLImportExport._2020_12.Briefcase import ImportObjectsFromBriefcaseInput, ExportObjectsToBriefcaseInput
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class BriefcaseService(TcService):

    @classmethod
    def importObjectsFromBriefcase(cls, input: ImportObjectsFromBriefcaseInput) -> ServiceData:
        """
        Imports the objects from a briefcase package file (.bcz) in asynchronous mode using the specified Transfer
        Option Set object and the additional options for scoping data. The import user will be notified with the import
        results when the import is completed.
        """
        return cls.execute_soa_method(
            method_name='importObjectsFromBriefcase',
            library='Internal-TCXMLImportExport',
            service_date='2020_12',
            service_name='Briefcase',
            params={'input': input},
            response_cls=ServiceData,
        )

    @classmethod
    def importObjectsFromBriefcaseAsync(cls, input: ImportObjectsFromBriefcaseInput) -> None:
        """
        Imports the objects from a Briefcase package in asynchronous mode using the specified Transfer Option Set
        object and the additional options for data scoping. The user will be notified with the import results when the
        import is completed.
        """
        return cls.execute_soa_method(
            method_name='importObjectsFromBriefcaseAsync',
            library='Internal-TCXMLImportExport',
            service_date='2020_12',
            service_name='Briefcase',
            params={'input': input},
            response_cls=None,
        )

    @classmethod
    def exportObjectsToBriefcase(cls, input: ExportObjectsToBriefcaseInput) -> ServiceData:
        """
        Exports the objects to the target site into a briefcase package file (.bcz) in asynchronous mode using the
        specified Transfer Option Set object and the additional options for scoping data. The export user will be
        notified with the export resultant data when the export is completed.
        """
        return cls.execute_soa_method(
            method_name='exportObjectsToBriefcase',
            library='Internal-TCXMLImportExport',
            service_date='2020_12',
            service_name='Briefcase',
            params={'input': input},
            response_cls=ServiceData,
        )

    @classmethod
    def exportObjectsToBriefcaseAsync(cls, input: ExportObjectsToBriefcaseInput) -> None:
        """
        Exports the objects to the target site into a briefcase package file (.bcz) in asynchronous mode using the
        specified Transfer Option Set object and the additional options for scoping data. The export user will be
        notified with the export resultant data when the export is completed.
        """
        return cls.execute_soa_method(
            method_name='exportObjectsToBriefcaseAsync',
            library='Internal-TCXMLImportExport',
            service_date='2020_12',
            service_name='Briefcase',
            params={'input': input},
            response_cls=None,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import RevisionRule, Cfg0SearchCursor, Cfg0ConfiguratorPerspective, Cfg0AbsFamilyGroup, Cfg0AbsOptionFamily, ItemRevision
from tcsoa.gen.Internal.Configurator._2014_06.ConfiguratorManagement import SearchRecipe, VariantRuleInput, SearchResponse, ValidateProductConfigurationResponse, ProductDefaultsInput, GetVariantCacheInfoResponse, GetOptionFamiliesResponse, ConfigExpressionDisplayStringInput, SearchOptions, ValidateProductConfigInput, FilterCriteria, GetFamilyGroupResponse, GetProductDefaultsResponse, CreateUpdateVariantRulesResponse, GetDisplayStringResponse, GetOptionValuesResponse
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class ConfiguratorManagementService(TcService):

    @classmethod
    def createUpdateVariantRules(cls, rules: List[VariantRuleInput], revRule: RevisionRule) -> CreateUpdateVariantRulesResponse:
        """
        This operation created or update the variant criteria object. Any object which has variant behavior implemented
        can be used as an variant criteria object. If parameter 'saveRule' is set as FALSE then variant criteria object
        is not saved and transient object is returned. The relationship name used to associate the variant criteria
        object to the product Item.
        
        Use cases:
        This operation can be used for following cases
        
        - Create new persistent variant criteria object.
        - Create new transient variant criteria object by providing 'saveRule' as False.
        - Create new and attach variant criteria object to the given product item by relation as mentioned in
        'relationName' or if passed empty then by default relationship TC_Default_SVR_Relationship.
        - Update variant expressions of the existsing variant criteria by providing existsing object in 'ruleToUpdate'.
        
        """
        return cls.execute_soa_method(
            method_name='createUpdateVariantRules',
            library='Internal-Configurator',
            service_date='2014_06',
            service_name='ConfiguratorManagement',
            params={'rules': rules, 'revRule': revRule},
            response_cls=CreateUpdateVariantRulesResponse,
        )

    @classmethod
    def getDefaultRules(cls, context: Cfg0ConfiguratorPerspective, filterCriteria: FilterCriteria) -> ServiceData:
        """
        This internal service operation returns the default rule instances as per the specified product items(s) in
        'context' and filter criteria. 
        The parameter ''filterCriteria'' allows filtering the rules returned by this operation based on criteria.
        Note: In Teamcenter 10.1.2 the product model(s) member in context is not supported.
        
        
        Use cases:
        This operation can be used to retrieve rules for the given product item(s) or product model(s) from parameter
        context. Parameter filterCriteria can be used for filtering based on specific family group(s) or family
        (families) or feature(s). 
        Consider that following set data:
        
        Family Groups: 
        Engine-Box - It has families "Engine" and "Transmission". 
        Engine - Petrol, Diesel
        Transmission - Manual, Auto
        Wheel - It has families "Wheel-drive" and "Suspension". 
        Wheel-drive - 2-Wheels, 4-Wheels
        Suspension - Full-Thrust, Full-Boom
        
        The rules are created as follows 
        
        1.If Transmission=Manual Then Set Wheel-drive=2-Wheels. 
        2.If Transmission=Auto Then Set Wheel-drive=4-Wheels. 
        3.If Engine=Diesel And Transmission=Manual Then Set Suspension=Full-Thrust. 
        4.If Engine=Petrol And Transmission=Manual Then Set Suspension=Full-Boom. 
        
        The response of operation for filter criteria will be as follows:
        Criteria: { Group: Engine-Box, Family:-, feature:-} - Rules returned will be 1,2,3,4. 
        Criteria: { Group: Engine-Box, Family: Engine, feature:-} - Rules returned will be 3 and 4. 
        Criteria: { Group: Engine-Box, Family: Engine, feature:Diesel} - Rules returned will be 3. 
        Criteria: { Group: -, Family: -, feature: Manual} - Rules returned will be 1,3,4.
        
        Exceptions:
        >No error conditions result at this time for this operation.
        """
        return cls.execute_soa_method(
            method_name='getDefaultRules',
            library='Internal-Configurator',
            service_date='2014_06',
            service_name='ConfiguratorManagement',
            params={'context': context, 'filterCriteria': filterCriteria},
            response_cls=ServiceData,
        )

    @classmethod
    def getExcludeRules(cls, context: Cfg0ConfiguratorPerspective, filterCriteria: FilterCriteria, severities: List[str]) -> ServiceData:
        """
        This internal service operation returns the exclusion rule instances based on the specified 'context', the
        filter criteria and rule severity. 
        The parameter ''filterCriteria'' allows filtering the rules returned by this operation based on criteria.
        Note: In Teamcenter 10.1.2, the product model(s) member in context is not supported.
        
        
        Use cases:
        This operation can be used to retrieve rules for the given product item(s) or product model(s) from parameter
        context. Parameter filterCriteria can be used for filtering based on specific family group(s) or family
        (families) or feature(s). 
        Consider that following set data: 
        Groups: 
              - Engine-Box - It has families "Engine" and "Transmission". 
              - Engine - Petrol, Diesel
        Transmission - Manual, Auto
              - Wheel - It has families "Wheel-drive" and "Suspension". 
              - Wheel-drive - 2-Wheels, 4-Wheels
        Suspension - Full-Thrust, Full-Boom
        
        
        The rules are created as follows 
        1. If Transmission=Manual And Set Wheel-drive=4-Wheels Error "Incompatible combination". 
        2. If Transmission=Auto And Set Wheel-drive=4-Wheels Info "Optimum configuration". 
        3. If Engine=Diesel And Transmission=Manual Warning "Suspension should be Full-Thrust". 
        4. If Engine=Petrol And Transmission=Manual Warning "Suspension should be Full-Boom". 
        
        The response of operation for filter criteria will be as follows 
        Criteria: { Group: Engine-Box, Family:-, Feature:-} - Rules returned will be 1,2,3,4. 
        Criteria: { Group: Engine-Box, Family: Engine, Feature:-} - Rules returned will be 3 and 4. 
        Criteria: { Group: Engine-Box, Family: Engine, Feature:Diesel} - Rules returned will be 3. 
        Criteria: { Group: -, Family: -, Feature: Manual} - Rules returned will be 1,3,4.
        
        Exceptions:
        >No error conditions result at this time for this operation.
        """
        return cls.execute_soa_method(
            method_name='getExcludeRules',
            library='Internal-Configurator',
            service_date='2014_06',
            service_name='ConfiguratorManagement',
            params={'context': context, 'filterCriteria': filterCriteria, 'severities': severities},
            response_cls=ServiceData,
        )

    @classmethod
    def getFamilyGroups(cls, context: Cfg0ConfiguratorPerspective) -> GetFamilyGroupResponse:
        """
        This internal operation returns the family group instances. The parameter context may provide either the
        product item(s). 
        If the product item(s) of context is available then those instances of the family group ('Cfg0AbsFamilyGroup'
        or subtypes) are returned which are allocated to the given product item(s). 
        
        
        
        Use cases:
        User can query family groups allocated to the product item(s) mentioned in 'context' parameter.
        
        Exceptions:
        >No error conditions result at this time for this operation.
        """
        return cls.execute_soa_method(
            method_name='getFamilyGroups',
            library='Internal-Configurator',
            service_date='2014_06',
            service_name='ConfiguratorManagement',
            params={'context': context},
            response_cls=GetFamilyGroupResponse,
        )

    @classmethod
    def getIncludeRules(cls, context: Cfg0ConfiguratorPerspective, filterCriteria: FilterCriteria, severities: List[str]) -> ServiceData:
        """
        This internal service operation returns the inclusion rule instances based on the specified context, the filter
        criteria and rule severity. 
        The parameter ''filterCriteria'' allows filtering the rules returned by this operation based on criteria.
        Note: In Teamcenter 10.1.2, the product model(s) member in context is not supported.
        
        
        Use cases:
        This operation can be used to retrieve rules for the given product item(s) or product model(s) from parameter
        'context'. Parameter 'filterCriteria' can be used for filtering based on specific family group(s) or family
        (families) or feature(s). 
        Consider that following set data: 
        Groups: 
        Engine-Box - It has families "Engine" and "Transmission". 
        - Engine - Petrol, Diesel
        - Transmission - Manual, Auto
        
        
        
        Wheel - It has families "Wheel-drive" and "Suspension". 
        - Wheel-drive - 2-Wheels, 4-Wheels
        - Suspension - Full-Thrust, Full-Boom
        
        
        
        The rules are created as follows 
        1. If Transmission=Manual And Set Wheel-drive=4-Wheels Error "Incompatible combination". 
        2. If Transmission=Auto And Set Wheel-drive=4-Wheels Info "Optimum configuration". 
        3. If Engine=Diesel And Transmission=Manual Warning "Suspension should be Full-Thrust". 
        4. If Engine=Petrol And Transmission=Manual Warning "Suspension should be Full-Boom". 
        
        The response of operation for filter criteria will be as follows 
        Criteria: { Group: Engine-Box, Family:-, Feature:-} - Rules returned will be 1,2,3,4. 
        Criteria: { Group: Engine-Box, Family: Engine, Feature:-} - Rules returned will be 3 and 4. 
        Criteria: { Group: Engine-Box, Family: Engine, Feature:Diesel} - Rules returned will be 3. 
        Criteria: { Group: -, Family: -, Feature: Manual} - Rules returned will be 1,3,4.
        
        Exceptions:
        >No error conditions result at this time for this operation.
        """
        return cls.execute_soa_method(
            method_name='getIncludeRules',
            library='Internal-Configurator',
            service_date='2014_06',
            service_name='ConfiguratorManagement',
            params={'context': context, 'filterCriteria': filterCriteria, 'severities': severities},
            response_cls=ServiceData,
        )

    @classmethod
    def getModelsForProduct(cls, context: Cfg0ConfiguratorPerspective, leafLevelOnly: bool) -> ServiceData:
        """
        This internal operation returns the product model instances for the given product item(s) in parameter context. 
        If the parameter ''leafLevelOnly'' is set to true, then all the product models (or subtypes) are returned and
        the summary models (or subtypes) are ignored. 
        
        
        Use cases:
        This operation can be used to retrieve the models for the product item(s).
        
        The product item "Car" has following option families and product models LXI,VXI,LDI and VDI in model family
        "Models". The product item also has summary models created "Petrol Models" and "Diesel Models" in summary model
        family "Summaries".
        
        User can use operation 'getModelsForProduct' as follows:
        
        If user wants to get all models i.e. product models and summary models, the parameter 'leafLevelOnly' should be
        passed as FALSE.
        
        If user wants get only product models i.e. LXI,VXI,LDI and VDI, the parameter 'leafLevelOnly' should be passed
        as TRUE.
        
        
        
        
        
        Exceptions:
        >No error conditions result at this time for this operation.
        """
        return cls.execute_soa_method(
            method_name='getModelsForProduct',
            library='Internal-Configurator',
            service_date='2014_06',
            service_name='ConfiguratorManagement',
            params={'context': context, 'leafLevelOnly': leafLevelOnly},
            response_cls=ServiceData,
        )

    @classmethod
    def getOptionFamilies(cls, context: Cfg0ConfiguratorPerspective, groups: List[Cfg0AbsFamilyGroup], otherFamilies: bool, unavailableFamilies: bool) -> GetOptionFamiliesResponse:
        """
        This internal operation returns the option family instances given 'context' information. If the product item(s)
        member is provided in context, then the option families that are allocated to any of the specified product
        item(s) are returned. If the product models member is provided in 'context', then the option families that are
        made available to at least one of given the product model(s) are returned.
        
        In addition, to filter the option families further, the ''groups'' input parameter can be leveraged. If this
        parameter is populated with the family group instances or IDs, then the option families that are allocated to
        any of the specified product item(s) and which are members of the specified family group instances are returned.
        If the parameter ''otherFamilies'' is set to 'true', then operation will return that option families which are
        not the members of any family group instances however are allocated to at least one of the specified product
        item(s).
        The default value for this parameter is 'false', which guides the system to only return the Option Families
        from the Family Groups specified in the input parameter 'groups'. 
        If  parameter ''unavailableFamilies'' is set to true, option families those are the  members of any of the
        specified groups and are allocated to specified product context are returned even if their availability rule is
        in an 'unavailable' state. 
        
        Note:  In Teamcenter 10.1.2 the product model(s) member of context is ignored.
        
        Use cases:
        This operation can be used to retrieve families for the given product item(s) or product model(s) from
        parameter 'context'. Parameter 'filterCriteria' can be used for filtering based on specific family group(s) or
        family (families) or feature(s). 
        
        Consider that following set data: 
        Groups: 
        Engine-Box - It has families "Engine" and "Transmission". 
        - Engine- Petrol, Diesel
        - Transmission - Manual, Auto
        
        
        
        Wheel - It has families "Wheel-drive" and "Suspension". 
        - Wheel-drive - 2-Wheels, 4-Wheels
        - Suspension - Full-Thrust, Full-Boom
        
        
        
        
        The response of operation for filter criteria will be as follows 
        Criteria: { Group: Engine-Box, Family:-, Value:-} - Option families returned will be Engine and Transmission. 
        Criteria: { Group: Engine-Box, Family: Engine, Value:-} - Option families returned will be Engine.
        
        Exceptions:
        >No error conditions result at this time for this operation.
        """
        return cls.execute_soa_method(
            method_name='getOptionFamilies',
            library='Internal-Configurator',
            service_date='2014_06',
            service_name='ConfiguratorManagement',
            params={'context': context, 'groups': groups, 'otherFamilies': otherFamilies, 'unavailableFamilies': unavailableFamilies},
            response_cls=GetOptionFamiliesResponse,
        )

    @classmethod
    def executeSearch(cls, searchRecipe: SearchRecipe, searchOptions: SearchOptions) -> SearchResponse:
        """
        Executes a search based on a search recipe. The search results can contain any sub-types of POM_Object. The
        search recipe could be a complex search definition that combines multiple simpler search definitions in a
        logical sequence, e.g. multiple saved queries as defined in the Teamcenter application QueryBuilder.
        
        The search is executed in the context of a configurator perspective (Cfg0ConfiguratorPerspective). Configurator
        objects such as features, or constraint rules, are filtered based on their relationship to the product context
        items and product models in the configurator perspective.
        
        Variant and effecitvity configurable objects in the search results are filtered according to the criteria
        associated with the VariantRule and RevisionRule in the configurator perspective.
        
        Revisable objects in the search results are revision configured according to the RevisionRule in the
        configurator perspective.
        
        The results of a search are returned in batches based on the 'defaultLoadCount' specified in the search
        options. A search cursor object is returned in the response with which subsequent batches can be requested with
        operation 'fetchNextSearchResults'.
        
        Search options may be used to sort the results of a search using one or more attributes of the returned objects.
        
        Use cases:
        1. Get models for product. Provide product context, revision rule and filter list in parameter
        'searchRecipe'.'configuratorPerspecitve', 'searchRecipe'.'searchExpression'.'savedQueries' contains a saved
        query for searching models.
        
        2. Get Family Groups. Provide product context, revision rule and filter list in parameter
        'searchRecipe'.'configuratorPerspective'; 'searchRecipe'.'searchExpression'.'savedQueries' contains a saved
        query for searching Family Groups.
        
        3. Get Option Families. Provide product context, revision rule and filter list in parameter
        'searchRecipe'.'configuratorPerspecitve'; 'searchRecipe'.'searchExpression'.'savedQueries' contains a saved
        query for searching families. 
        
        4. Get Option Values. Provide product context, revision rule and filter list in parameter
        'searchRecipe'.'configuratorPerspecitve'; 'searchRecipe'.'searchExpression'.'savedQueries' will have a saved
        query for searching features.
        
        5. Get Derived Default Rules. Provide product context, revision rule and filter list in parameter
        'searchRecipe.configuratorPerspecitve; searchRecipe.searchExpression.savedQueries' will have a saved query for
        searching derived default rules, 'searchRecipe'.'searchExpression'.'filterCriteria' will have the filter
        information.
        
        6. Get Exclude Rules. Provide product context, revision rule and filter list in parameter
        'searchRecipe.configuratorPerspecitve; searchRecipe.searchExpression.savedQueries' will have a saved query for
        searching exclude rules, 'searchRecipe'.'searchExpression'.'filterCriteria' will have the filter information.
        
        7. Get Include Rules. Provide product context, revision rule and filter list in parameter
        'searchRecipe.configuratorPerspecitve; searchRecipe.searchExpression.savedQueries' will have a saved query for
        searching include rules,'searchRecipe.searchExpression.filterCriteria' will have the filter information.
        
        Exceptions:
        >No error conditions result at this time for this operation.
        """
        return cls.execute_soa_method(
            method_name='executeSearch',
            library='Internal-Configurator',
            service_date='2014_06',
            service_name='ConfiguratorManagement',
            params={'searchRecipe': searchRecipe, 'searchOptions': searchOptions},
            response_cls=SearchResponse,
        )

    @classmethod
    def getOptionValues(cls, context: Cfg0ConfiguratorPerspective, groups: List[Cfg0AbsFamilyGroup], families: List[Cfg0AbsOptionFamily], unavailableValues: bool) -> GetOptionValuesResponse:
        """
        This internal service operation returns the feature instances based on the specified 'context' and other
        parameters. If product item(s) member is provided in the 'context', then the features that are allocated to any
        of the product items are returned.  If product model(s) member is provided in the context, then the features
        that are made available to any of the product models are returned.
        If 'groups' parameter is populated with the family group instances, then only those feature instances which
        belong to the family instances of given list of family group members are returned.
        If 'families' parameter is populated with the family instances, then only those feature instances which belong
        to the given family instances are returned.
        If 'unavailableValues' is set to 'true', it would also return the features which are marked with negative
        availability. 
        
        Note: In Teamcenter 10.1.2 the product model(s) member in context is not considered. This member is used for
        availability feature and this feature will be supported in Teamcenter 10.1.3 onwards.
        
        Use cases:
        This operation can be used to retrieve features for the given product item(s) or product model(s) from
        parameter 'context'. Parameter 'filterCriteria' can be used for filtering based on specific family group(s) or
        family (families) or feature(s). 
        
        Consider that following set data: 
        Groups: 
        Engine-Box - It has families "Engine" and "Transmission". 
        - Engine- Petrol, Diesel
        - Transmission - Manual, Auto
        
        
        
        Wheel - It has families "Wheel-drive" and "Suspension". 
        - Wheel-drive - 2-Wheels, 4-Wheels
        - Suspension - Full-Thrust, Full-Boom
        
        
        
        
        The response of operation for filter criteria will be as follows 
        Criteria: { Group: Engine-Box, Family:-, Feature:-} - Features returned will be Petrol, Diesel, Manual and
        Auto. 
        Criteria: { Group: Engine-Box, Family: Engine, Feature:-} - Features returned will be Petrol and Diesel.
        
        Exceptions:
        >No error conditions result at this time for this operation.
        """
        return cls.execute_soa_method(
            method_name='getOptionValues',
            library='Internal-Configurator',
            service_date='2014_06',
            service_name='ConfiguratorManagement',
            params={'context': context, 'groups': groups, 'families': families, 'unavailableValues': unavailableValues},
            response_cls=GetOptionValuesResponse,
        )

    @classmethod
    def getProductDefaults(cls, inputs: List[ProductDefaultsInput]) -> GetProductDefaultsResponse:
        """
        Gets the results of applying default rules to a set of input expressions in the context of the associated
        Cfg0ConfiguratorPerspective objects.
        
        Use cases:
        The 'getProductDefaults' operation can be used in following ways: 
        
        Example 1: System has default rule which states "If TYPE=CLASSIC and ENGINE=V4 then Apply COLOR=RED" 
        
        If user provides input as "TYPE=CLASSIC and ENGINE=V4" then response provided by operation as  "TYPE=CLASSIC
        and ENGINE=V4 and COLOR=RED" in  'expressionsWithDefaults'.
        
        To filter response based on option families 
        
        Example 2: System has default rule which states "If TYPE=CLASSIC and ENGINE=V4 then Apply COLOR=RED"
        
        If user provides input expression to getProductDefaults SOA as TYPE=CLASSIC and ENGINE=V4
        Input filter of Families is provided as TYPE AND ENGINE AND COLOR
        
        Then output of SOA in expressionsWithDefaults is expected as TYPE=CLASSIC and ENGINE=V4 and COLOR=RED
        
        Example 3: System has default rule which states "If TYPE=CLASSIC and ENGINE=V4 then Apply COLOR=RED"
        
        If user provides input expression to getProductDefaults SOA as "TYPE=CLASSIC and ENGINE=V4"
        Input filter of Families is provided as TRANSMISSION
        
        Then output of SOA in expressionsWithDefaults is expected same as input. Because the family provided in filter
        does not match with any of the term in updated expression.
        
        Exceptions:
        >No error conditions result at this time for this operation.
        """
        return cls.execute_soa_method(
            method_name='getProductDefaults',
            library='Internal-Configurator',
            service_date='2014_06',
            service_name='ConfiguratorManagement',
            params={'inputs': inputs},
            response_cls=GetProductDefaultsResponse,
        )

    @classmethod
    def getVariantCache(cls, productItem: ItemRevision, revRule: RevisionRule, productModelUID: str, productItemUID: str) -> GetVariantCacheInfoResponse:
        """
        This operation is designed to download revision configured variant data from a higher Teamcenter release at
        real time. Use this operation if data model changes in a higher Teamcenter release make it impossible to use
        offline data replication. The local replica of the product context ItemRevision will contain a stub
        VariantExpressionBlock.
        
        Use cases:
        'GetVariantCache' can be invoked to fetch variant data such as families, features and rules.
        
        Exceptions:
        >No error conditions result at this time for this operation.
        """
        return cls.execute_soa_method(
            method_name='getVariantCache',
            library='Internal-Configurator',
            service_date='2014_06',
            service_name='ConfiguratorManagement',
            params={'productItem': productItem, 'revRule': revRule, 'productModelUID': productModelUID, 'productItemUID': productItemUID},
            response_cls=GetVariantCacheInfoResponse,
        )

    @classmethod
    def getVariantExpressionDisplayStrings(cls, inputs: List[ConfigExpressionDisplayStringInput], revRule: RevisionRule) -> GetDisplayStringResponse:
        """
        This operation converts the input variant expressions of boolean expression format into displayable or user
        readable format.
        
        Use cases:
        This operation can be used to display formulae for 'ConfigExpression' structures returned by
        'getVariantExpressions' operation. The examples of display formulae are as follows:
        (NS is family namespace)
        
        1)    Boolean expression
        -  [NS]BOOL - reprepsents TRUE
        -  [NS]!BOOL - reprepsents FALSE
        
        
        
        2)    Date expressions
        -  [NS]REL_DATE = 22-Jan-2013 00:00      (dd-mmm-yyyy  HH:SS)
        
        
        
        3)    Range expressions
        -  [NS]A = a1 & [NS]LENGTH <= 50
        -  [NS]A = a1 & ([NS]LENGTH >= 1 & [NS]LENGTH <= 10)
        -  [NS]A = a1 & [NS]LENGTH >= 20
        -  [NS]A = a1 & !([NS]LENGTH >= 20)
        
        
        
        4)    Multiple column expressions must be grouped within brackets
        -  ([NS]A = a1 & [NS]B = b1) | ([NS]A = a2 & [NS]C = c1)
        
        
        
        5)    Multiple selections within a single column
        - ([NS]A = a1 | [NS]A = a2) & [NS]B = b1
        
        
        
        6)    Multiple selections in a family 'C' that has Multi-select=Yes
        - ([NS]A = a1 | [NS]A = a2) & [NS]C = c1 & [NS]C = c2
        
        
        
        7)    Optional family 'D' [Target]
        -  [NS]A = a1 & [NS]D = NONE (same as [NS]D = '')
        -  [NS]A = a1 & [NS]D = ANY (same as [NS]D != '')
        
        """
        return cls.execute_soa_method(
            method_name='getVariantExpressionDisplayStrings',
            library='Internal-Configurator',
            service_date='2014_06',
            service_name='ConfiguratorManagement',
            params={'inputs': inputs, 'revRule': revRule},
            response_cls=GetDisplayStringResponse,
        )

    @classmethod
    def fetchNextSearchResults(cls, searchCursor: Cfg0SearchCursor, loadCount: int) -> SearchResponse:
        """
        This operation gets the next set of objects in the search result of a previously executed search. The returned
        results will be based on the load count specified in the SearchCursor input structure.
        
        Use cases:
        This API is used in conjunction with 'executeSearch' operation. 'executeSearch' operation is a prerequisite for
        invoking 'fetchNextSearchResults'. The search cursor returned by the 'executeSearch' is used to call
        'fetchNextSearchResults' operation. This operation could be called repeatedly by the caller, until all the
        search results are returned.
        
        Exceptions:
        >No error conditions result at this time for this operation.
        """
        return cls.execute_soa_method(
            method_name='fetchNextSearchResults',
            library='Internal-Configurator',
            service_date='2014_06',
            service_name='ConfiguratorManagement',
            params={'searchCursor': searchCursor, 'loadCount': loadCount},
            response_cls=SearchResponse,
        )

    @classmethod
    def stopSearch(cls, searchCursor: Cfg0SearchCursor) -> ServiceData:
        """
        This operation stops further loading of objects from a previously executed search and clears all the memory
        allocated for the search operation. It deletes the search cursor and the list of  Objects that are kept track
        by the Search cursor from the Server Memory. 
        
        The 'stopSearch' does not unload any previously loaded objects. Also there is no locking or unlocking performed
        by the 'stopSearch' operation. 
        
        
        Use cases:
        When a search is executed in Advanced Configurator and the search returns a large set of objects. The server
        process keeps the results of a search using a search cursor object. At each 'fetchNextSearchResults' operation,
        the results are further filtered and returned in batches specified by the load count. However if the caller is
        not interested in consuming the search results further, then a 'stopSearch' could be called to release all the
        resources allocated for that search operation. This includes dropping the runtime search cursor object and the
        list of Configurator Objects kept track by the cursor.
        
        Exceptions:
        >No error conditions result at this time for this operation.
        """
        return cls.execute_soa_method(
            method_name='stopSearch',
            library='Internal-Configurator',
            service_date='2014_06',
            service_name='ConfiguratorManagement',
            params={'searchCursor': searchCursor},
            response_cls=ServiceData,
        )

    @classmethod
    def validateProductConfiguration(cls, inputs: List[ValidateProductConfigInput]) -> ValidateProductConfigurationResponse:
        """
        Validates the given expression by applying the constraint rules and returns violations and an updated
        expression resulting from the application of include rules that can be set on the expression.
        
        The value of TC_Default_Solve_Type preference is used for using solve type for this operation.
        
        Values for 'applyConstraints' are as integrated as follows
         1 //enable configurator constraint evaluation
        + 8 //report warnings in addition to errors
        + 16 //also report informational messages
        + 32 //don't stop if a violation is encountered
        + 256 //skip the check against _allocated_ variability which could raise a 
            "k_variant_criteria_outside" validation failure
        + 1024 //skip criteria update based on validation rules (For example configurator exclusion rule). If this flag
        is not set then the operation will update the variant criteria after applying validation rules.
        + 2048 //enable Availability Constraint evaluation.
        
        
        Use cases:
        This' 'operation validates the given variant expression for the product item.
        
        Use case 1:
        Consider that user has created exclusion rule which states If TYPE=CLASSIC and ENGINE=V4 And COLOR=RED then
        Error "COLOR cannot be RED for CLASSIC V4 model". 
        
        Now if user provides input expression as "TYPE=CLASSIC and ENGINE=V4 and COLOR=RED" to
        validateProductConfiuration operation to validate the expression. Then in output of the operation a violation
        is expected with message as "COLOR cannot be RED for CLASSIC V4 model". 
        
        Use Case 2:
        Consider that user has created an availability say 'RED' is available to model 'LUXURY'. User has not defined
        availability of 'BLUE' for model 'LUXURY'.
        Now, if availability constraint evaluation is enabled and user provides an expression as "MODEL=LUXURY and
        COLOR=BLUE", then in the output of the operation a violation is expected with message as "The Feature "BLUE" is
        not available to the following product models: LUXURY".
        
        Exceptions:
        >No error conditions result at this time for this operation.
        """
        return cls.execute_soa_method(
            method_name='validateProductConfiguration',
            library='Internal-Configurator',
            service_date='2014_06',
            service_name='ConfiguratorManagement',
            params={'inputs': inputs},
            response_cls=ValidateProductConfigurationResponse,
        )



from __future__ import annotations

from tcsoa.gen.Cad._2007_01.DataManagement import ItemInfo, ExtraObjectInfo, ItemRevInfo, DatasetFileInfo, AttributeInfo
from tcsoa.gen.BusinessObjects import BusinessObject, CadAttrMappingDefinition, ItemRevision, Dataset
from typing import Dict, List
from tcsoa.gen.Cad._2007_12.DataManagement import NamedReferenceObjectInfo2
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class MappedDatasetAttrPropertyInfo(TcBaseObj):
    """
    Contains resolved object and property name along with CadAttrMappingDefinition object reference associated with the
    resolved property object.
    
    :var cadAttrMappingDefinition: CadAttrMappingDefinition object reference associated with the resolved property
    Object.
    :var resolvedObject: Object reference of object holding mapped attribute value
    :var resolvedPropertyName: The property name of the mapped object holding the attribute value of interest resulting
    from evaluation of a dataset CAD attribute mapping definition.
    """
    cadAttrMappingDefinition: CadAttrMappingDefinition = None
    resolvedObject: BusinessObject = None
    resolvedPropertyName: str = ''


@dataclass
class PartInfo3(TcBaseObj):
    """
    The PartInfo3 struct is the main input to the createOrUpdateParts service.
    This structure refers to the Item, ItemRevision, and one or more Dataset structures used to create those objects.
    
    :var clientId: Identifier defined by user to track the related object.
    :var itemInput: Member of type ItemInfo
    :var itemRevInput: Member of type ItemRevInfo
    :var datasetInput: List of DatasetInfo3
    """
    clientId: str = ''
    itemInput: ItemInfo = None
    itemRevInput: ItemRevInfo = None
    datasetInput: List[DatasetInfo3] = ()


@dataclass
class ResolveAttrMappingsInfo(TcBaseObj):
    """
    Contains dataset, item revision and list of 'CadAttrMappingDefinition' object references to use to resolve the
    mapping.
    
    :var dataset: Dataset object reference to use as starting point to get mapped attribute values.
    :var itemRev: ItemRevision object reference helps resolve ambiguity in the mapping traversal for the dataset or can
    be the starting point for the traversal as well.
    :var mappingDefinitionInfos: List of CadAttrMappingDefinitionInfo objects
    """
    dataset: Dataset = None
    itemRev: ItemRevision = None
    mappingDefinitionInfos: List[CadAttrMappingDefinitionInfo] = ()


@dataclass
class ResolveAttrMappingsResponse(TcBaseObj):
    """
    Holds the response for resolveAttrMappings. The processing of the input is as follows :
    1.    Process the ResolveAttrMappingsInfo first. This will validate the dataset and item revision inputs are valid.
    If this validation fails, then an error will be returned with the index of the ResolveMappinsInfo being the
    identifier with the error. 
    2.    Process the list of CadAttrMappingDefinitionInfo. If this results in an error then the identifier for the
    error will be the clientId specified in the CadAttributeMappingDefinition.
    When processing the output, if a key with thte clientId is not found, the application should first look for an
    error with an identifier of the cleintId. If no error is found then the index of the input that contained the
    clientId should be found.
    
    
    :var resolvedMappingsMap: Member of type 'ResolveAttrMappingsOutputMap'. This is a map containing the successfully
    mapped property information. The keys are the input 'clientIds' and the values are the output
    'MappedDatasetAttrPropertyInfo' structures.
    :var serviceData: Service data contains any partial errors and exceptions. The objects holding the mapped
    attributes, resulting from successfully resolved mappings, are returned as plain objects. The mapped attribute
    properties are returned as 'ServiceData' properties.
    """
    resolvedMappingsMap: ResolveAttrMappingsOutputMap = None
    serviceData: ServiceData = None


@dataclass
class CadAttrMappingDefinitionInfo(TcBaseObj):
    """
    Contains unique 'clientId' and 'CadAttributeMappingDefinition' object reference associated with the resolved
    property object.
    
    :var clientId: Unique client side identifier. This is a required input parameter If the 'ClientId' is not provided
    or not unique a partial error is reported and the property for the particular 'CADAttrMappingDefinition' will not
    resolved.
    :var cadAttrMappingDefinition: CadAttributeMappingDefinition object reference associated with the resolved property
    object
    """
    clientId: str = ''
    cadAttrMappingDefinition: CadAttrMappingDefinition = None


@dataclass
class DatasetInfo3(TcBaseObj):
    """
    The DatasetInfo3 struct represents all of the data necessary to construct the dataset object.
    The basic attributes that are required are passed as named elements in the struct.
    All other attributes are passed as name/value pairs in the AttributeInfo struct.
    The extraObject field allows for the creation of an object(s) that will be related to this newly created Dataset.
    
    :var clientId: Identifier defined by user to track the related object.
    :var dataset: Dataset object reference for update, null for creation
    :var createNewVersion: Flag to create new version ( TRUE ) or not (FALSE )
    :var mapAttributesWithoutDataset: Flag to indicate whether DatasetInfo should be used for mapping attributes or for
    create.
    :var namedReferencePreference: Preference name which holds the list of named references to delete from one Dataset
    version to the next.
    :var attrList: List of AttributeInfos for attributes
    :var mappingAttributes: List of AttributeInfos for mapped attributes. Mapped atributes are attributes that are
    applied to other objects. Refere to the ITK manual for a definition of attribute mapping.
    :var extraObject: List of ExtraObjectInfos
    :var datasetFileInfos: List of DatasetFileInfos
    :var namedReferenceObjectInfos: List of NamedReferenceObjectInfos
    :var name: Name attribute value
    :var basisName: Basis Name attribute value, used when the name is null or blank
    :var description: Description attribute value
    :var type: Type attribute value
    :var lastModifiedOfDataset: Last Modified Date of dataset
    :var id: ID attribute value
    :var datasetRev: Revision attribute value
    :var itemRevRelationName: Required input, may not be null, not defaulted
    """
    clientId: str = ''
    dataset: Dataset = None
    createNewVersion: bool = False
    mapAttributesWithoutDataset: bool = False
    namedReferencePreference: str = ''
    attrList: List[AttributeInfo] = ()
    mappingAttributes: List[AttributeInfo] = ()
    extraObject: List[ExtraObjectInfo] = ()
    datasetFileInfos: List[DatasetFileInfo] = ()
    namedReferenceObjectInfos: List[NamedReferenceObjectInfo2] = ()
    name: str = ''
    basisName: str = ''
    description: str = ''
    type: str = ''
    lastModifiedOfDataset: datetime = None
    id: str = ''
    datasetRev: str = ''
    itemRevRelationName: str = ''


"""
ResolveAttrMappingsOutputMap
"""
ResolveAttrMappingsOutputMap = Dict[str, MappedDatasetAttrPropertyInfo]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class VendorPartProperties2(TcBaseObj):
    """
    The information about each ManufacturerPart to be processed is provided by way of the VendorPartProperties2 data
    structure.
    
    :var partId: Id of ManufacturerPart to be created, if not specified or blank then operation will generate the
    partId.
    :var partName: Part Name of the ManufacturerPart to be created.
    :var partType: Object type to be created. Only "ManufacturerPart" is valid type.
    :var revId: Revision Id specified for create. Generated if the string is empty.
    :var vendorObj: Vendor object to be associated with ManufacturerPart.
    :var companyLocationObj: CompanyLocation object to be associated with ManufacturerPart. If null, then no location
    will be associated with ManufacturerPart object.
    :var clientId: A unique string supplied by the caller. This ID is used to identify the Partial Errors associated
    with input.
    """
    partId: str = ''
    partName: str = ''
    partType: str = ''
    revId: str = ''
    vendorObj: BusinessObject = None
    companyLocationObj: BusinessObject = None
    clientId: str = ''



from __future__ import annotations

from tcsoa.gen.Rdv._2008_05.ContextManagement import AddPartSolutionResponse, GetProductItemResponse, RemoveABEPartsInputInfo, ReplacePartSolutionResponse, GetRemoveABEPartsResponse, ReplacePartSolutionInputInfo, AddPartSolutionInputInfo
from typing import List
from tcsoa.base import TcService


class ContextManagementService(TcService):

    @classmethod
    def removePartsRelatedToABE(cls, inputs: List[RemoveABEPartsInputInfo]) -> GetRemoveABEPartsResponse:
        """
        Deletes all the related Part Solutions of an Architecture Breakdown Element (ABE). The links through which the
        Part solutions are related to the Architecture Breakdown Element are also removed.
        
        Use cases:
        The 'removePartsRelatedToABE' operation is called when user wants to remove all part solutions related to
        Architecture Breakdown Element and their corresponding links. The user can specify the input Architecture
        Breakdown Element and top line to which the Architecture Breakdown Element is linked using
        'RemoveABEPartsInputInfo' object.
        """
        return cls.execute_soa_method(
            method_name='removePartsRelatedToABE',
            library='Rdv',
            service_date='2008_05',
            service_name='ContextManagement',
            params={'inputs': inputs},
            response_cls=GetRemoveABEPartsResponse,
        )

    @classmethod
    def replacePartInProduct(cls, inputs: List[ReplacePartSolutionInputInfo]) -> ReplacePartSolutionResponse:
        """
        get the required Information for replace part in product
        """
        return cls.execute_soa_method(
            method_name='replacePartInProduct',
            library='Rdv',
            service_date='2008_05',
            service_name='ContextManagement',
            params={'inputs': inputs},
            response_cls=ReplacePartSolutionResponse,
        )

    @classmethod
    def addPartToProduct(cls, inputs: List[AddPartSolutionInputInfo]) -> AddPartSolutionResponse:
        """
        get the required Information for add part to product
        """
        return cls.execute_soa_method(
            method_name='addPartToProduct',
            library='Rdv',
            service_date='2008_05',
            service_name='ContextManagement',
            params={'inputs': inputs},
            response_cls=AddPartSolutionResponse,
        )

    @classmethod
    def getProductItemInfo(cls) -> GetProductItemResponse:
        """
        Returns a list of all Product Items found in the database. The following preferences can be used to define an
        object as a Product Item. 
        The preference PortalDesignContextProductItemProperties is used to specify one or more of the following
        properties to be used to define the object as Product Item. Multiple properties could be specified at the same
        time and the algorithm will check all the specified property values to be satisfied in order to deem an object
        as a Product Item.
        -     object_type
        -     object_desc
        -     owning_group
        
        
        For Example: PortalDesignContextProductItemProperties = object_type,object_desc
        The preference PortalDesignContextProductItemProperties.<property_name>  is used to specify the value to be
        checked in order to determine whether the object is a Product Item. For example:
        PortalDesignContextProductItemProperties.object_type = CORP_Vehicle
        Only an object of type Item or its sub type could be defined as a Product Item. The operation will return empty
        if any other objects are specified.
        The default values are
        PortalDesignContextProductItemProperties = object_type, object_desc
        PortalDesignContextProductItemProperties.object_type=Item
        PortalDesignContextProductItemProperties.object_desc=Product
        
        Use cases:
        The 'getProductItemInfo' operation is called when user wants to fetch all Product Items present in the database
        which are defined by setting the PortalDesignContextProductItemProperties preference.
        """
        return cls.execute_soa_method(
            method_name='getProductItemInfo',
            library='Rdv',
            service_date='2008_05',
            service_name='ContextManagement',
            params={},
            response_cls=GetProductItemResponse,
        )



from __future__ import annotations

from typing import List
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class PrecalRoutineInfo(TcBaseObj):
    """
    The PrecalRoutineInfo structure is used give the results of the service that calculates percentage on target
    
    :var routineDate: Date of execution of the events
    :var percentOnTarget: Percentage of attributes on target for that day
    """
    routineDate: str = ''
    percentOnTarget: float = 0.0


@dataclass
class PrecalRoutineInfoResponse(TcBaseObj):
    """
    The PrecalRoutineInfoResponse structure is used to return the calculated values
    
    :var result: A vector of PrecalRoutineInfo structures. Each structure corresponding to a date in the given date
    time range
    """
    result: List[PrecalRoutineInfo] = ()



from __future__ import annotations

from tcsoa.gen.Internal.IcsAw._2017_06.Author import ClassificationObject, CreateOrUpdateClsObjectsResponse
from typing import List
from tcsoa.base import TcService


class AuthorService(TcService):

    @classmethod
    def createOrUpdateClassificationObjects(cls, classificationObjects: List[ClassificationObject]) -> CreateOrUpdateClsObjectsResponse:
        """
        Creates or updates one or more classification objects and (optionally) attach them to a WorkspaceObject, thus
        classifying it. When the Classification objects are not associated with any WorkspaceObject, they would act as
        standalone classification objects. A classification object is also called ICO.
        
        Use cases:
        User wants to classify a workspace object or create a standalone classification object (ICO) or update an
        existing classification object in Active Workspace. This operation can be combined with other operations like
        createItems() to create workspace object and then associate the workspace object to the classification object.
        Before creating a classification object, a classification class hierarchy should already be created by the
        classification admin user in Teamcenter. This hierarchy should include a storage class (a class that allows
        instances to be created and associated to it) for which the classification objects need to be created. Values
        of any attributes associated with classification objects can also be populated.
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateClassificationObjects',
            library='Internal-IcsAw',
            service_date='2017_06',
            service_name='Author',
            params={'classificationObjects': classificationObjects},
            response_cls=CreateOrUpdateClsObjectsResponse,
        )
